//****************************************************************************
// Commodore 64 verilog computer. Asynchonous simulation architecture
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

//`define debug
//`define la
//`define mouser

module Atlys64A(clk100

                , LED

                , SW

                , BTN_RESET
                , BTN_LEFT
                , BTN_RIGHT
                , BTN_UP
                , BTN_DOWN
                , BTN_CENTER

                , HSYNC, VSYNC, RED, GREEN, BLUE
                , LP

                , AUDIOL, AUDIOR

                , nFLAG1, CNTAin, SPAin, SPAout, nPC1, CNTAout, PAin
                , nFLAG2, CNTBin, SPBin, SPBout, nPC2, CNTBout
                , PA_Out2, PAout2_7, PAout2_6

//adept io - virtual pport
                , EppAstb
                , EppDstb
                , EppWr
                , EppWait
                , EppDB
                    
                , RS232_RX_P, RS232_RX_N
                , RS232_TX_P, RS232_TX_N
              
`ifdef mouser
                , MOUSECLOCK_P, MOUSEDATA_P
                , MOUSECLOCK_N, MOUSEDATA_N
`endif

                , KBCLOCK_P, KBDATA_P
                , KBCLOCK_N, KBDATA_N
               );

//#################################################          
// io definitions
//
input clk100;

output [7:0] LED;

input [7:0] SW;

input BTN_RESET;
input BTN_LEFT;
input BTN_RIGHT;
input BTN_UP;
input BTN_DOWN;
input BTN_CENTER;

input LP;
output HSYNC, VSYNC;
output [2:0] GREEN, BLUE;
output [1:0] RED;

output AUDIOL, AUDIOR;

input EppAstb, EppDstb, EppWr;
output EppWait;
inout [7:0] EppDB;
  
input [5:0] PAin;
input nFLAG1, nFLAG2, CNTAin, CNTBin, SPAin, SPBin;
output PA_Out2, PAout2_7, PAout2_6;
output SPAout, SPBout;
output nPC1, nPC2, CNTAout, CNTBout;

input KBCLOCK_P, KBDATA_P;
input KBCLOCK_N, KBDATA_N;

`ifdef mouser
input MOUSECLOCK_P, MOUSEDATA_P;
input MOUSECLOCK_N, MOUSEDATA_N;
`endif

input RS232_RX_P, RS232_RX_N;
output RS232_TX_P, RS232_TX_N;

parameter H = 1'b1;
parameter L = 1'b0;

diffIn io0(RS232_RX_P, RS232_RX_N, RS232_RX);
diffOut io1(RS232_TX, RS232_TX_P, RS232_TX_N);

`ifdef mouser
diffIn io2(MOUSECLOCK_P, MOUSECLOCK_N, MOUSECLOCK);
diffIn io3(MOUSEDATA_P, MOUSEDATA_N, MOUSEDATA);
`endif

diffIn io4(KBCLOCK_P, KBCLOCK_N, KBCLOCK);
diffIn io5(KBDATA_P, KBDATA_N, KBDATA);

//#################################################          
// bus definitions
`ifdef debug
wire [47:0] debug_Data; //cpu internal register display
wire [15:0] break_pc;
wire [4:0] debug_Address;
wire [2:0] debug_sel;
`endif

`ifdef la
wire [7:0] la_offset;
`endif

`ifdef mouser
wire [47:0] dAB; //decoded address bus (for periph registers)
`else
wire [46:0] dAB; //decoded address bus (for periph registers)
`endif
wire [31:0] cpu_ir, basc, krnl, char, ram, vic_bitmap;
wire [15:0] cpu_a, dma_a, vic_color2;
wire [15:1] vic_a;
wire [15:2] cpu_al;
wire [11:0] externalV;
`ifdef mouser
wire [8:0] mouse_x;
wire [7:0] mouse_y;
`endif
wire [7:0] cpu_pw, dma_dw, cpu_mem_rw;
wire [7:0] DBRvic, DBRcia1, DBRcia2, DBRsid;
wire [7:0] PAin2, PAout2, PBout1, PBout2;
wire [7:0] COLout, ROWin, keyCode, ascii;
wire [7:0] paddleA, paddleB, cpu_br, dma_dr;
wire [7:0] cpu_a_periph_io1, cpu_pw_io1, DBRio1, x_mag, send, byteIO;
wire [7:0] cpu_a_periph_io2, cpu_pw_io2, DBRio2, y_mag, receive;
wire [4:0] laCounter;
wire [3:0] cpu_byte_enable, DBRcolor;

reg [15:0] vic_color;
reg [7:0] LED;
reg toggle; // alive signal

`ifdef la
assign la_enable    = SW[7]; //active low, la_arm 0x24
`endif
`ifdef mouser
assign mouse_enable = SW[6]; //active low
`endif
//assign mouse_sample = SW[5]; //active low, normally high
//assign unused = SW[5]; //active low, normally high
assign cpu_EXROM    = SW[4]; //normally high
assign cpu_GAME     = SW[3]; //normally high
//assign vic_dma_dis  = SW[2]; //active high, only high with livedesign memory config on atlys board 
`ifdef debug
assign debug_sel[2] = SW[2]; // 3'b111 == normal display
assign debug_sel[1] = SW[1];
assign debug_sel[0] = SW[0];
`endif

//#################################################          
// master clock signals
//clock_generator clocks(clk100, sysclk, mclk, vclk, cclk, TOD, sclk);
clock_generator_2DCM clocks(clk100, sysclk, mclk, vclk, cclk, TOD, sclk);

//#################################################          
// cpu
X6510A cpu(mclk,
           cpu_a, cpu_al, cpu_ir, pRead,
           cpu_br, cpu_pw, cpu_rw, cpu_byte_enable,
           vic_req, vic_ack, vic_a,
           dma_req, dma_ack, dma_a, dma_dw, dma_rw,
           (nmi & restoreKey), (irq & vic_irq),
           cpu_reset, cpu_pause,
           cpu_hiram, cpu_loram, cpu_charen
`ifdef debug
           ,debug_Address, debug_Data, debug_sel, breakTrig, break_pc, cpu_ack
`endif
          );

//#################################################          
// glue logic
plaA glue(mclk,
          cpu_ir, cpu_br,
          cpu_a[15:8], cpu_a[5:0],
          dAB,
          ram, krnl, basc, char,
          DBRvic, DBRsid, DBRcia1, DBRcia2, DBRio1, DBRio2, DBRcolor,
          (vic_req | vic_ack), vic_charRom, vic_bitmap,
          dma_ack, dma_dr, dma_ack3, dma_GAME, dma_EXROM, dma_hiram, dma_loram, dma_charen,
          cpu_GAME, cpu_EXROM, cpu_hiram, cpu_loram, cpu_charen, cpu_rw,
          vic_w_reg, sid_w_reg, cia1_w_reg, cia2_w_reg, io1_w_reg, io2_w_reg,
          vic_r_reg, cia1_r_reg, cia2_r_reg, io1_r_reg, io2_r_reg,
          color_ram_w, ramselw, pRead
          );
          
//#################################################          
// roms
kernel KrnlRom(~mclk, cpu_al[12:2], cpu_a[12:2], krnl);
char   CharRom(~mclk, cpu_al[11:2], cpu_a[11:2], char);
basic  BascRom(~mclk, cpu_al[12:2], cpu_a[12:2], basc);

//#################################################          
// 64K ram memory
mem16Kx2Byte RamH(~mclk, ramselw, cpu_byte_enable[3:2],  cpu_a[15:2], { cpu_pw, cpu_pw }, ram[31:16]);
mem16Kx2Byte RamL(~mclk, ramselw, cpu_byte_enable[1:0], cpu_al[15:2], { cpu_pw, cpu_pw }, ram[15:0]);

//#################################################          
// color ram - vic reads simultaneously with system ram
color_ram color(~mclk, color_ram_w, cpu_a[9:0], cpu_pw[3:0], DBRcolor, cpu_a[9:2], vic_color2);

//#################################################          
// use sysclk for something other than the logic analyzer
always@(posedge sysclk)
  begin
`ifdef mouser
    LED[7] <= (BTN_LEFT | left | shiftLout);
    LED[6] <= (BTN_RIGHT | right | shiftRout);
    LED[3] <=  BTN_CENTER | center;
`else
    LED[7] <= (BTN_LEFT | shiftLout);
    LED[6] <= (BTN_RIGHT | shiftRout);
    LED[3] <=  BTN_CENTER;
`endif
    LED[5] <=  BTN_UP;
    LED[4] <=  BTN_DOWN;
    LED[2] <=  toggle ^ (io1_w_reg | io2_w_reg | io1_r_reg | io2_r_reg);
    LED[1] <= ~BTN_RESET;
    LED[0] <=  (^SW) ^ toggle;
  end

//#################################################          
// data read selection
always@(posedge mclk)
  begin
    if(sclk) toggle <= ~toggle;
    
    vic_color <= vic_color2;
  end

//#################################################          
// video controller
X6567 vic(mclk, vclk, RED, GREEN, BLUE, VSYNC, HSYNC, cpu_reset
          , dAB, vic_w_reg, vic_r_reg, DBRvic, cpu_pw
          , vic_a, vic_bitmap, vic_color, vic_req, vic_charRom
          , PAout2[1], PAout2[0]
          , LP, vic_irq
`ifdef mouser
          , mouse_enable ? 9'h1FF : mouse_x, mouse_y
`endif
`ifdef la
          , laCounter, la_sync, ~lain, samplingDone
`endif
`ifdef debug
          , debug_Address, debug_Data, debug_sel
`endif
          );


//#################################################          
// sound controller
X6581 sid(mclk, dAB[28:0], sid_w_reg, DBRsid, cpu_pw, cpu_reset, cclk,
          AUDIOL, AUDIOR, paddleA, paddleB, externalV);

assign paddleA = PBout1;
assign paddleB = PBout2;
assign externalV = dma_a[15:4];

//assign AUDIOL = 1;
//assign AUDIOR = 1;

//#################################################          
// cia1 controller
X6526 cia1(mclk, dAB[15:0], cia1_w_reg, cia1_r_reg, DBRcia1, cpu_pw, cpu_reset, cclk,
           keyCode, COLout, ROWin, PBout1,
           CNTAin, SPAin, CNTAout, SPAout,
           TOD, nPC1, nFLAG1, irq);

//#################################################          
// cia2 controller
X6526 cia2(mclk, dAB[15:0], cia2_w_reg, cia2_r_reg, DBRcia2, cpu_pw, cpu_reset, cclk,
           PAin2, PAout2, keyCode, PBout2,
           CNTBin, SPBin, CNTBout, SPBout,
           TOD, nPC2, nFLAG2, nmi);

////#################################################          
////io1
//assign cpu_a_periph_io1 = (~io1_w_reg | ~io1_r_reg | mouse_sample) ? cpu_a2 : x_mag; 
//assign cpu_pw_io1 = (~io1_w_reg | ~io1_r_reg | mouse_sample) ? cpu_pw2 : DBRio1p1; 
//mem256Byte io1(mclk, (~io1_w_reg | ~io1_r_reg | mouse_sample) ? io1_w_reg : changeX2, cpu_a_periph_io1, cpu_pw_io1, DBRio1);
assign DBRio1 = 8'hBE;

////#################################################          
////io2
//assign cpu_a_periph_io2 = (~io2_w_reg | ~io2_r_reg | mouse_sample) ? cpu_a2 : y_mag; 
//assign cpu_pw_io2 = (~io2_w_reg | ~io2_r_reg | mouse_sample) ? cpu_pw2 : DBRio2p1; 
//mem256Byte io2(mclk, (~io2_w_reg | ~io2_r_reg | mouse_sample) ? io2_w_reg : changeY2, cpu_a_periph_io2, cpu_pw_io2, DBRio2);
assign DBRio2 = 8'hEF;

//#################################################          
// keyboard control
ps2keyboard kbd(mclk, KBCLOCK, KBDATA, cclk, cpu_reset,
                ROWin, COLout, restoreKey, keyCode, shiftLout, shiftRout);//, key_pause, key_reset);

`ifdef mouser
//#################################################          
// mouse control
ps2mouse mouse(mclk, MOUSECLOCK, MOUSEDATA, cclk, cpu_reset,
               mouse_x, mouse_y, left, right, center);
`endif
                
//#################################################          
// pport address control - 1284 interface to adept
IOExpansion pport(mclk, EppAstb, EppDstb, EppWr, EppDB, EppWait, LED,
                  dma_a, dma_dw, dma_dr, dma_ack, dma_ack3,
                  dma_req, dma_rw, dma_hiram, dma_loram, dma_charen, dma_GAME, dma_EXROM,
                  cpu_reset, cpu_pause
`ifdef la
                  ,sampleclk, clear_trigger, la_offset
`endif
`ifdef debug
                  ,breakTrig, break_pc, cpu_ack
`endif
                  );

//#################################################          
// serial port
rs232 serial(mclk, cpu_reset, RS232_RX, RS232_TX,// RS232_RX_CLK, RS232_TX_CLK,
             valid, receive, clear, eoiIn, //ackByte, //talker,
             busy, send, xfer, atnOut, eoiOut//,
             //sClkIn, sDataIn, sClkOut, sDataOut, sAtnOut
             //, rxState, txState
             //, iecState
             );

//#################################################          
// serial bus control
IEC sport(mclk, cpu_reset, vic_req,
          sClkOut, sDataOut, sAtnOut, 
          sClkIn, sDataIn, 
          busy, xfer, send, eoiOut, atnOut,
			    clear, valid, receive, eoiIn//, ackByte //, talker 
//          , iecState, byteIO
          );

assign sDataOut = ~PAout2[5];
assign sClkOut = ~PAout2[4];
assign sAtnOut = ~PAout2[3];
assign PA_Out2 = PAout2[2];
assign PAout2_7 = PAout2[7];
assign PAout2_6 = PAout2[6];
assign PAin2[7] = sDataOut & sDataIn;
assign PAin2[6] = sClkOut & sClkIn;
assign PAin2[5:0] = PAin[5:0];

`ifdef la
reg [1:0] divclk;

always@(posedge mclk)
  begin
    divclk <= divclk + H;
  end
  
assign sampleclk = sysclk;
//assign sampleclk = mclk;
//assign sampleclk = divclk[0];
//assign sampleclk = vclk;
//assign sampleclk = cclk;

//#################################################          
// logic analyzer
//                trigger* & manual trigger
logic_analyzer la(dma_req,
//                sample clk,me mclk,vid clk,arm,         enable,sync with video,datastream,lamemfull,pixelcounter,memoffset
                  sampleclk, mclk, vclk, clear_trigger, la_enable, la_sync, lain, samplingDone, laCounter, la_offset
//                  , din[31:0]
//                  , { sAtnOut, sClkOut, sDataOut, L, sClkIn, sDataIn, L, RS232_RX_CLK, RS232_RX, L, RS232_TX_CLK, RS232_TX
//                  , io1_w_reg, io1_r_reg, io2_w_reg, io2_r_reg
//                  , L, busy, xfer, eoiOut, atnOut
//                  , L, clear, valid, eoiIn, talker
//                  , iecState}
                  , { dma_req, dma_ack, mclk, cpu_a, mclk, dma_dr, mclk, cpu_rw, ramselw, pRead }
//                  , { dma_req, dma_ack, mclk, cpu_a, mclk, krnl[7:0], mclk, cpu_rw, ramselw, pRead }
`endif

endmodule
