//****************************************************************************
// 8 bit binary alu.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

//mode == 0 -> add
//mode == 1 -> sub (or cmp)
module accumulatorPlus(mode, Ain, Bin, Cin, Dout, Cout, D_zero, D_negative, D_overflow,
                       B_0, B_1, B_255, B_dec_neg, B_inc_neg,
                       asl_op, rol_op, lsr_op, ror_op, dec_op, inc_op,
                       or_op, and_op, eor_op, or_op_zero, and_op_zero, eor_op_zero);

input [7:0] Ain, Bin;
input mode, Cin;

output [7:0] Dout, asl_op, rol_op, lsr_op, ror_op, dec_op, inc_op, or_op, and_op, eor_op;
output Cout, D_zero, D_negative, D_overflow, B_0, B_1, B_255, B_dec_neg, B_inc_neg, or_op_zero, and_op_zero, eor_op_zero;

parameter ZERO = 8'h00;
parameter ONE = 8'h01;
parameter NEG_ONE = 8'hFF;
parameter H = 1'b1;
parameter L = 1'b0;

accumulator Acc(mode, Ain[7:0], Bin[7:0], Cin, Dout[7:0], Cout, D_zero, D_negative, D_overflow, B_0);

assign B_1 = ~Bin[7] & ~(|Bin[6:1]) & Bin[0];
assign B_63 = &Bin[6:0];
assign B_255 = Bin[7] & B_63;
assign B_dec_neg = (Bin[7] & (|Bin[6:0])) | B_0;
assign B_inc_neg = (Bin[7] | B_63) & ~B_255;

assign asl_op = { Bin[6:0], L   };
assign rol_op = { Bin[6:0], Cin };
assign lsr_op = { L, Bin[7:1]   };
assign ror_op = { Cin, Bin[7:1] };
assign dec_op = Bin - ONE;
assign inc_op = Bin + ONE;

assign or_op = Ain | Bin;
assign or_op_zero = (or_op == ZERO);
assign and_op = Ain & Bin;
assign and_op_zero = (and_op == ZERO);
assign eor_op = Ain ^ Bin;
assign eor_op_zero = (eor_op == ZERO);

endmodule
