//****************************************************************************
// 6502 8 bit BCD add/sub.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

//mode == 0 -> add
//mode == 1 -> sub (or cmp)
module bcdAccumulator(mode, Ain, Bin, Cin, Dout, Cout, D_zero, D_negative, D_overflow);

input [7:0] Ain, Bin;
input mode, Cin;

output [7:0] Dout;
output Cout, D_zero, D_negative, D_overflow;

wire [6:0] carrys;
wire [7:0] bdout;

// binary addition and subtraction accumulator
adder add_bit0(mode, Ain[0], Bin[0],       Cin, bdout[0], carrys[0]);
adder add_bit1(mode, Ain[1], Bin[1], carrys[0], bdout[1], carrys[1]);
adder add_bit2(mode, Ain[2], Bin[2], carrys[1], bdout[2], carrys[2]);
adder add_bit3(mode, Ain[3], Bin[3], carrys[2], bdout[3], carrys[3]);
adder add_bit4(mode, Ain[4], Bin[4], carrys[3], bdout[4], carrys[4]);
adder add_bit5(mode, Ain[5], Bin[5], carrys[4], bdout[5], carrys[5]);
adder add_bit6(mode, Ain[6], Bin[6], carrys[5], bdout[6], carrys[6]);
adder add_bit7(mode, Ain[7], Bin[7], carrys[6], bdout[7], bcout);

wire [2:0] dcarrys;
wire [6:4] d2carrys;
wire [6:0] dcarrys3;
wire [7:0] ddout;
wire [7:0] tdout, tdout3;
wire [3:0] dcarrys2, tdout2;

// for bcd subtraction - decimal adjust low nybble
adder add_dbit0(1'b1, bdout[0], 1'b1,       1'b0, tdout[0], dcarrys[0]);
adder add_dbit1(1'b1, bdout[1], 1'b0, dcarrys[0], tdout[1], dcarrys[1]);
adder add_dbit2(1'b1, bdout[2], 1'b1, dcarrys[1], tdout[2], dcarrys[2]);
adder add_dbit3(1'b1, bdout[3], 1'b0, dcarrys[2], tdout[3]);
// for bcd addition and subtraction
adder add_dbit4(mode, Ain[4], Bin[4], mode ? dcarrys2[3] : halfCarry, tdout[4], d2carrys[4]);
adder add_dbit5(mode, Ain[5], Bin[5],                     d2carrys[4], tdout[5], d2carrys[5]);
adder add_dbit6(mode, Ain[6], Bin[6],                     d2carrys[5], tdout[6], d2carrys[6]);
adder add_dbit7(mode, Ain[7], Bin[7],                     d2carrys[6], tdout[7], dcout);

// for bcd subtraction
adder add_dsbit0(1'b1, carrys[3] ? bdout[0] : tdout[0], ~carrys[3],   carrys[3], tdout2[0], dcarrys2[0]);
adder add_dsbit1(1'b1, carrys[3] ? bdout[1] : tdout[1], ~carrys[3], dcarrys2[0], tdout2[1], dcarrys2[1]);
adder add_dsbit2(1'b1, carrys[3] ? bdout[2] : tdout[2], ~carrys[3], dcarrys2[1], tdout2[2], dcarrys2[2]);
adder add_dsbit3(1'b1, carrys[3] ? bdout[3] : tdout[3], ~carrys[3], dcarrys2[2], tdout2[3], dcarrys2[3]);

// for bcd subtraction - decimal adjust high nybble
adder add_ds2bit0(1'b1, tdout2[0], 1'b1,       1'b0,  tdout3[0], dcarrys3[0]);
adder add_ds2bit1(1'b1, tdout2[1], 1'b1, dcarrys3[0], tdout3[1], dcarrys3[1]);
adder add_ds2bit2(1'b1, tdout2[2], 1'b1, dcarrys3[1], tdout3[2], dcarrys3[2]);
adder add_ds2bit3(1'b1, tdout2[3], 1'b1, dcarrys3[2], tdout3[3], dcarrys3[3]);
adder add_ds2bit4(1'b1,  tdout[4], 1'b1, dcarrys3[3], tdout3[4], dcarrys3[4]);
adder add_ds2bit5(1'b1,  tdout[5], 1'b0, dcarrys3[4], tdout3[5], dcarrys3[5]);
adder add_ds2bit6(1'b1,  tdout[6], 1'b1, dcarrys3[5], tdout3[6], dcarrys3[6]);
adder add_ds2bit7(1'b1,  tdout[7], 1'b0, dcarrys3[6], tdout3[7]);

// bcd (add) flag results
assign halfCarry = (bdout[3] & (|bdout[2:1])) | carrys[3];

// bcd accumulator results
assign ddout[3:0] = mode ? (dcout ? tdout2[3:0] : tdout3[3:0]) : (halfCarry ? (bdout[3:0] + 4'h6) : bdout[3:0]);
assign ddout[7:4] = mode ? (dcout ?  tdout[7:4] : tdout3[7:4]) :      (Cout ? (tdout[7:4] + 4'h6) : tdout[7:4]);

// final accumulator results
assign Dout = ddout;

//final flag results
assign Cout = ~mode ? ((tdout[7] & (|tdout[6:5])) | dcout) : bcout;
assign D_negative = ~mode ? tdout[7] : bdout[7];
assign D_overflow = ~mode ? d2carrys[6] ^ dcout : carrys[6] ^ bcout;

assign D_zero = ~(|bdout[7:0]);

endmodule
