//****************************************************************************
// CPU debug display selector.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

module cpu_debug(clk, pc_trace, ad_trace, aw_trace
             ,state,halt,cpu_rwz_r,cpu_byte_enablez,cache_rr,cpu_rw_r,cpu_byte_enable,a1
             ,negative,decimal,interrupt,overflow,zero,carry,rirq,nmi
             ,ir,iCount
             ,a,x,y,sp,alu_op,cpu_pr_r,cpu_wb_r
             ,pc,tmp_a,cpu_if,cpu_a2_i,cpu_al_i,cpu_aw2_i,break_pc
             , cpu_drr, cycleCount, cycleCount2
             ,debug_Address, debug_Dout, debug_sel
             );

input clk;
input [2:0] debug_sel;
input [4:0] debug_Address;
input [95:0] ir;

//debug trace registers
input [959:0] pc_trace;
input [959:0] ad_trace;
input [959:0] aw_trace;

input [3:0] state, iCount, cpu_byte_enablez, cpu_byte_enable, halt;
input [2:0] a1;
input negative,decimal,interrupt,overflow,zero,carry,rirq,nmi,cpu_rwz_r,cpu_rw_r;
input [7:0] a,x,y,sp,alu_op,cpu_pr_r,cpu_wb_r;
input [15:0] cpu_aw2_i,cpu_a2_i,pc,tmp_a,break_pc;
input [15:1] cpu_if;
input [31:0] cpu_drr, cache_rr, cycleCount, cycleCount2;
input [15:2] cpu_al_i;

output [47:0] debug_Dout;

parameter H = 1'b1;
parameter L = 1'b0;

//debug video characters
parameter c_0 = 6'h00;
parameter c_1 = 6'h01;
parameter c_2 = 6'h02;
parameter c_3 = 6'h03;
parameter c_4 = 6'h04;
parameter c_5 = 6'h05;
parameter c_6 = 6'h06;
parameter c_7 = 6'h07;
parameter c_8 = 6'h08;
parameter c_9 = 6'h09;
parameter c_A = 6'h0A;
parameter c_B = 6'h0B;
parameter c_C = 6'h0C;
parameter c_D = 6'h0D;
parameter c_E = 6'h0E;
parameter c_F = 6'h0F;
parameter c_G = 6'h10;
parameter c_H = 6'h11;
parameter c_I = 6'h12;
parameter c_J = 6'h13;
parameter c_K = 6'h14;
parameter c_L = 6'h15;
parameter c_M = 6'h16;
parameter c_N = 6'h17;
parameter c_O = 6'h18;
parameter c_P = 6'h19;
parameter c_Q = 6'h1A;
parameter c_R = 6'h1B;
parameter c_S = 6'h1C;
parameter c_T = 6'h1D;
parameter c_U = 6'h1E;
parameter c_V = 6'h1F;
parameter c_W = 6'h20;
parameter c_X = 6'h21;
parameter c_Y = 6'h22;
parameter c_Z = 6'h23;
parameter spc = 6'h24;
parameter neg = 6'h25;
parameter fwd = 6'h26;
parameter unk = 6'h27;

//cpu state machine state names

// put reset vector on address bus to read the startup pc
parameter S_RESET = 4'h0;
// load IR with first instruction bytes
parameter S_LOAD_INST = 4'h1;
// decode the first instruction of new code path
parameter S_DECODE_INST = 4'h2;
// execute 1 byte instruction
parameter S_EXECUTE_INST1 = 4'h3;
// execute 2 byte instruction
parameter S_EXECUTE_INST2 = 4'h4;
// execute 3 byte instruction
parameter S_EXECUTE_INST3 = 4'h5;
// indirect y operand base address load
parameter S_IND_Y = 4'h6;
// read memory write back operand
parameter S_RD_MEM_OP = 4'h7;
// write memory operand
parameter S_WR_MEM_OP = 4'h8;
// rts pc adjust (+1)
parameter S_REG_OP = 4'h9;
// give BCD arithmatic an extra cycle to complete
parameter S_BCD_DELAY = 4'hA;
// rts pc adjust (+1)
parameter S_RTS = 4'hB;
// process nmi
parameter S_NMI = 4'hC;
// process irq
parameter S_IRQ = 4'hD;
// reload instruction register - ir may be corrupted by self modifying code
parameter S_IR_W_HIT = 4'hE;
// load more instruction bytes
parameter S_IR_LOAD = 4'hF;


reg [47:0] debug_Dout;

always @(posedge clk)
  begin
    case(debug_sel)
    0:
      begin
        case(debug_Address)
      //n_??????
      0 : 
        begin
          debug_Dout[47:30] <= { 6'h00, { 2'h0, state }, spc }; //0n_
          case(state)
            S_RESET        : debug_Dout[29:0] <= { c_R, c_E, c_S, c_E, c_T }; //n_RESET
            S_LOAD_INST    : debug_Dout[29:0] <= { c_L, c_D, spc, c_I, spc }; //n_LD_I
            S_DECODE_INST  : debug_Dout[29:0] <= { c_D, c_E, c_C, spc, c_I }; //n_DEC_I
            S_EXECUTE_INST1: debug_Dout[29:0] <= { c_E, c_X, c_E, c_1, spc }; //n_EXE1
            S_EXECUTE_INST2: debug_Dout[29:0] <= { c_E, c_X, c_E, c_2, spc }; //n_EXE2
            S_EXECUTE_INST3: debug_Dout[29:0] <= { c_E, c_X, c_E, c_3, spc }; //n_EXE3
            S_IND_Y        : debug_Dout[29:0] <= { c_I, c_N, c_D, spc, c_Y }; //n_IND_Y
            S_RD_MEM_OP    : debug_Dout[29:0] <= { c_R, c_D, c_M, c_E, c_M }; //n_RDMEM
            S_WR_MEM_OP    : debug_Dout[29:0] <= { c_W, c_R, c_M, c_E, c_M }; //n_WRMEM
            S_REG_OP       : debug_Dout[29:0] <= { c_R, spc, c_O, c_P, spc }; //n_R_OP
            S_BCD_DELAY    : debug_Dout[29:0] <= { c_B, c_C, c_D, spc, spc }; //n_BCD
            S_RTS          : debug_Dout[29:0] <= { c_R, c_T, c_S, spc, spc }; //n_RTS
            S_NMI          : debug_Dout[29:0] <= { c_N, c_M, c_I, spc, spc }; //n_NMI
            S_IRQ          : debug_Dout[29:0] <= { c_I, c_R, c_Q, spc, spc }; //n_IRQ
            S_IR_W_HIT     : debug_Dout[29:0] <= { c_I, c_R, c_H, c_I, c_T }; //n_IRHIT
            S_IR_LOAD      : debug_Dout[29:0] <= { c_I, c_R, c_L, c_D, spc }; //n_IRLD
            default        : debug_Dout[29:0] <= { c_U, c_N, c_K, spc, spc }; //n_UNK
          endcase
        end
`ifndef N6502
      //ADD1_h__
      1 : debug_Dout <= { c_A, c_D, c_D, c_1, spc, { 3'h0, a1[2:0] }, spc, spc };
`endif
      //NV-BDIZC
      2 : debug_Dout <= { c_N, c_V, neg, c_B, c_D, c_I, c_Z, c_C }; 
      //NV__DIZC
      3 : debug_Dout <= { { 5'h00, negative }, { 5'h00, overflow }, neg, neg,
                          { 5'h00, decimal }, { 5'h00, interrupt }, { 5'h00, zero }, { 5'h00, carry } };
      //A_hh____
      4 : debug_Dout <= { c_A, spc, { 2'h0, a[7:4] }, { 2'h0, a[3:0] }, spc, spc, { 2'h0, alu_op[7:4] }, { 2'h0, alu_op[3:0] } };
      //X_hh____
      5 : debug_Dout <= { c_X, spc, { 2'h0, x[7:4] }, { 2'h0, x[3:0] }, spc, spc, spc, spc };
      //Y_hh____
      6 : debug_Dout <= { c_Y, spc, { 2'h0, y[7:4] }, { 2'h0, y[3:0] }, spc, spc, spc, spc };
      //SP_01hh_
      7 : debug_Dout <= { c_S, c_P, spc, spc, c_0, c_1, { 2'h0, sp[7:4] }, { 2'h0,sp[3:0] } };
      //PC_hhhh_
      8 : debug_Dout <= { c_P, c_C, spc, spc, { 2'h0, pc[15:12] }, { 2'h0, pc[11:8] }, { 2'h0, pc[7:4] }, { 2'h0, pc[3:0] } };
      //TMP_hhhh
      9 : debug_Dout <= { c_T, c_M, c_P, spc, { 2'h0, tmp_a[15:12] }, { 2'h0, tmp_a[11:8] }, { 2'h0, tmp_a[7:4] }, { 2'h0, tmp_a[3:0] } };
      //IF__hhhh
     10 : debug_Dout <= { c_I, c_F, spc, spc, { 2'h0, cpu_if[15:12] }, { 2'h0, cpu_if[11:8] }, { 2'h0, cpu_if[7:4] }, { 2'h0, cpu_if[3:1], L } };
      //ADD_hhhh
     11 : debug_Dout <= { c_A, c_D, c_D, spc, { 2'h0, cpu_a2_i[15:12] }, { 2'h0, cpu_a2_i[11:8] }, { 2'h0, cpu_a2_i[7:4] }, { 2'h0, cpu_a2_i[3:0] } };
      //ADL_hhhh
     12 : debug_Dout <= { c_A, c_D, c_L, spc, { 2'h0, cpu_al_i[15:12] }, { 2'h0, cpu_al_i[11:8] }, { 2'h0, cpu_al_i[7:4] }, { 2'h0, cpu_al_i[3:2], 2'h0 } };
      //BENh_RWh
     13 : debug_Dout <= { c_B, c_E, c_N, { 2'h0, cpu_byte_enable[3:0] },
                          spc, c_R, c_W, { 5'h00, cpu_rw_r } };
//      //DR/R2/DW
//     14 : debug_Dout <= { c_D, c_R, fwd, c_C, c_R, fwd, c_D, c_W };
      //hhhhhhhhh
     14 : debug_Dout <= { { 2'h0, cpu_drr[31:28] }, { 2'h0, cpu_drr[27:24] }, { 2'h0, cpu_drr[23:20] }, { 2'h0, cpu_drr[19:16] },
                          { 2'h0, cpu_drr[15:12] }, { 2'h0, cpu_drr[11:8] },  { 2'h0, cpu_drr[7:4] },   { 2'h0, cpu_drr[3:0] } };
      //PRhhPWhh
     16 : debug_Dout <= { c_P, c_R, { 2'h0, cpu_pr_r[7:4] }, { 2'h0, cpu_pr_r[3:0] }, c_W, c_B, { 2'h0, cpu_wb_r[7:4] }, { 2'h0, cpu_wb_r[3:0] } };
      //hhhhhhhhh
      //ADW_hhhh
     18 : debug_Dout <= { c_A, c_D, c_W, spc, { 2'h0, cpu_aw2_i[15:12] }, { 2'h0, cpu_aw2_i[11:8] }, { 2'h0, cpu_aw2_i[7:4] }, { 2'h0, cpu_aw2_i[3:0] } };

      //hhhhhhhhh
//     16 : debug_Dout <= { { 2'h0, cpu_dw_r[31:28] }, { 2'h0, cpu_dw_r[27:24] }, { 2'h0, cpu_dw_r[23:20] }, { 2'h0, cpu_dw_r[19:16] },
//                          { 2'h0, cpu_dw_r[15:12] }, { 2'h0, cpu_dw_r[11:8] },  { 2'h0, cpu_dw_r[7:4] },   { 2'h0, cpu_dw_r[3:0] } };
      //BENh_RWh
     19 : debug_Dout <= { c_B, c_E, c_Z, { 2'h0, cpu_byte_enablez[3:0] },
                          c_R, c_W, c_Z, { 5'h00, cpu_rwz_r } };
//     20 : debug_Dout <= { { 2'h0, ib3[7:4] }, { 2'h0, ib3[3:0] }, { 2'h0, ib2[7:4] }, { 2'h0, ib2[3:0] },
//                          { 2'h0, ib1[7:4] }, { 2'h0, ib1[3:0] }, { 2'h0, ib0[7:4] }, { 2'h0, ib0[3:0] } };
     20 : debug_Dout <= { { 2'h0, cache_rr[31:28] }, { 2'h0, cache_rr[27:24] }, { 2'h0, cache_rr[23:20] }, { 2'h0, cache_rr[19:16] },
                          { 2'h0, cache_rr[15:12] }, { 2'h0, cache_rr[11:8] },  { 2'h0, cache_rr[7:4] },   { 2'h0, cache_rr[3:0] } };
     //IFChIFRh
     //20 : debug_Dout <= { c_I, c_F, c_C, { 5'h00, ifCycle }, c_I, c_F, c_R, { 5'h00, ifReady } };
      //ICh_____
     21 : debug_Dout <= { c_I, c_C, { 2'h0, iCount[3:0] }, spc, spc, spc, c_I, c_R };
      //IRhhhhh
     22 : debug_Dout <= { { 2'h0, ir[95:92] }, { 2'h0, ir[91:88] }, { 2'h0, ir[87:84] }, { 2'h0, ir[83:80] },
                          { 2'h0, ir[79:76] }, { 2'h0, ir[75:72] }, { 2'h0, ir[71:68] }, { 2'h0, ir[67:64] } };
      //hhhhhhh
     23 : debug_Dout <= { { 2'h0, ir[63:60] }, { 2'h0, ir[59:56] }, { 2'h0, ir[55:52] }, { 2'h0, ir[51:48] },
                          { 2'h0, ir[47:44] }, { 2'h0, ir[43:40] }, { 2'h0, ir[39:36] }, { 2'h0, ir[35:32] } };
      //hhhhhhh
     24 : debug_Dout <= { { 2'h0, ir[31:28] }, { 2'h0, ir[27:24] }, { 2'h0, ir[23:20] }, { 2'h0, ir[19:16] },
                          { 2'h0, ir[15:12] }, { 2'h0, ir[11:8] },  { 2'h0, ir[7:4] },   { 2'h0, ir[3:0] } };
      //CYCLCNT-
//     25 : debug_Dout <= { c_C, c_Y, c_C, c_L, c_C, c_N, c_T, neg };
      //hhhhhhh
     25 : debug_Dout <= { { 2'h0, cycleCount[31:28] }, { 2'h0, cycleCount[27:24] }, { 2'h0, cycleCount[23:20] }, { 2'h0, cycleCount[19:16] },
                          { 2'h0, cycleCount[15:12] }, { 2'h0, cycleCount[11:8] },  { 2'h0, cycleCount[7:4] },   { 2'h0, cycleCount[3:0] } };
      //hhhhhhh
     26 : debug_Dout <= { { 2'h0, cycleCount2[31:28] }, { 2'h0, cycleCount2[27:24] }, { 2'h0, cycleCount2[23:20] }, { 2'h0, cycleCount2[19:16] },
                          { 2'h0, cycleCount2[15:12] }, { 2'h0, cycleCount2[11:8] },  { 2'h0, cycleCount2[7:4] },   { 2'h0, cycleCount2[3:0] } };
//     25: debug_Dout <= { { 2'h0, cache_ww[31:28] }, { 2'h0, cache_ww[27:24] }, { 2'h0, cache_ww[23:20] }, { 2'h0, cache_ww[19:16] },
//                         { 2'h0, cache_ww[15:12] }, { 2'h0, cache_ww[11:8] },  { 2'h0, cache_ww[7:4] },   { 2'h0, cache_ww[3:0] } };
//     26: debug_Dout <= { { 2'h0, cache_rw[31:28] }, { 2'h0, cache_rw[27:24] }, { 2'h0, cache_rw[23:20] }, { 2'h0, cache_rw[19:16] },
//                         { 2'h0, cache_rw[15:12] }, { 2'h0, cache_rw[11:8] },  { 2'h0, cache_rw[7:4] },   { 2'h0, cache_rw[3:0] } };
      //HLThPRDh
     27 : debug_Dout <= { c_H, c_L, c_T, { 2'b00, halt[3:0] },
                          spc, spc, spc, spc };
      //NMIhIRQh
     28 : debug_Dout <= { c_N, c_M, c_I, { 5'h00, nmi },
                          c_I, c_R, c_Q, { 5'h00, rirq } };
      //hhhhhhhh
     29 : debug_Dout <= { c_B, c_R, c_K, spc, { 2'h0, break_pc[15:12] }, { 2'h0, break_pc[11:8] }, { 2'h0, break_pc[7:4] }, { 2'h0, break_pc[3:0] } };
       //TR_hhhh_
     //30 : debug_Dout <= { c_T, c_R, spc, spc, { 4'h0, trace_a[13:12] }, { 2'h0, trace_a[11:8] }, { 2'h0, trace_a[7:4] }, { 2'h0, trace_a[3:0] } };

          default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
        endcase
      end


   1:
      begin
        case(debug_Address)
          0 : debug_Dout <= { { 2'h0, pc_trace[31:28] },   { 2'h0, pc_trace[27:24] },   { 2'h0, pc_trace[23:20] },   { 2'h0, pc_trace[19:16]   },
                              { 2'h0, pc_trace[15:12] },   { 2'h0, pc_trace[11:8] },    { 2'h0, pc_trace[7:4] },     { 2'h0, pc_trace[3:0]     } };
          1 : debug_Dout <= { { 2'h0, pc_trace[63:60] },   { 2'h0, pc_trace[59:56] },   { 2'h0, pc_trace[55:52] },   { 2'h0, pc_trace[51:48]   },
                              { 2'h0, pc_trace[47:44] },   { 2'h0, pc_trace[43:40] },   { 2'h0, pc_trace[39:36] },   { 2'h0, pc_trace[35:32]   } };
          2 : debug_Dout <= { { 2'h0, pc_trace[95:92] },   { 2'h0, pc_trace[91:88] },   { 2'h0, pc_trace[87:84] },   { 2'h0, pc_trace[83:80]   },
                              { 2'h0, pc_trace[79:76] },   { 2'h0, pc_trace[75:72] },   { 2'h0, pc_trace[71:68] },   { 2'h0, pc_trace[67:64]   } };
          3 : debug_Dout <= { { 2'h0, pc_trace[127:124] }, { 2'h0, pc_trace[123:120] }, { 2'h0, pc_trace[119:116] }, { 2'h0, pc_trace[115:112] },
                              { 2'h0, pc_trace[111:108] }, { 2'h0, pc_trace[107:104] }, { 2'h0, pc_trace[103:100] }, { 2'h0, pc_trace[99:96]   } };
          4 : debug_Dout <= { { 2'h0, pc_trace[159:156] }, { 2'h0, pc_trace[155:152] }, { 2'h0, pc_trace[151:148] }, { 2'h0, pc_trace[147:144] },
                              { 2'h0, pc_trace[143:140] }, { 2'h0, pc_trace[139:136] }, { 2'h0, pc_trace[135:132] }, { 2'h0, pc_trace[131:128] } };
          5 : debug_Dout <= { { 2'h0, pc_trace[191:188] }, { 2'h0, pc_trace[187:184] }, { 2'h0, pc_trace[183:180] }, { 2'h0, pc_trace[179:176] },
                              { 2'h0, pc_trace[175:172] }, { 2'h0, pc_trace[171:168] }, { 2'h0, pc_trace[167:164] }, { 2'h0, pc_trace[163:160] } };
          6 : debug_Dout <= { { 2'h0, pc_trace[223:220] }, { 2'h0, pc_trace[219:216] }, { 2'h0, pc_trace[215:212] }, { 2'h0, pc_trace[211:208] },
                              { 2'h0, pc_trace[207:204] }, { 2'h0, pc_trace[203:200] }, { 2'h0, pc_trace[199:196] }, { 2'h0, pc_trace[195:192] } };
          7 : debug_Dout <= { { 2'h0, pc_trace[255:252] }, { 2'h0, pc_trace[251:248] }, { 2'h0, pc_trace[247:244] }, { 2'h0, pc_trace[243:240] },
                              { 2'h0, pc_trace[239:236] }, { 2'h0, pc_trace[235:232] }, { 2'h0, pc_trace[231:228] }, { 2'h0, pc_trace[227:224] } };
          8 : debug_Dout <= { { 2'h0, pc_trace[287:284] }, { 2'h0, pc_trace[283:280] }, { 2'h0, pc_trace[279:276] }, { 2'h0, pc_trace[275:272] },
                              { 2'h0, pc_trace[271:268] }, { 2'h0, pc_trace[267:264] }, { 2'h0, pc_trace[263:260] }, { 2'h0, pc_trace[259:256] } };
          9 : debug_Dout <= { { 2'h0, pc_trace[319:316] }, { 2'h0, pc_trace[315:312] }, { 2'h0, pc_trace[311:308] }, { 2'h0, pc_trace[307:304] },
                              { 2'h0, pc_trace[303:300] }, { 2'h0, pc_trace[299:296] }, { 2'h0, pc_trace[295:292] }, { 2'h0, pc_trace[291:288] } };
         10 : debug_Dout <= { { 2'h0, pc_trace[351:348] }, { 2'h0, pc_trace[347:344] }, { 2'h0, pc_trace[343:340] }, { 2'h0, pc_trace[339:336] },
                              { 2'h0, pc_trace[335:332] }, { 2'h0, pc_trace[331:328] }, { 2'h0, pc_trace[327:324] }, { 2'h0, pc_trace[323:320] } };
         11 : debug_Dout <= { { 2'h0, pc_trace[383:380] }, { 2'h0, pc_trace[379:376] }, { 2'h0, pc_trace[375:372] }, { 2'h0, pc_trace[371:368] },
                              { 2'h0, pc_trace[367:364] }, { 2'h0, pc_trace[363:360] }, { 2'h0, pc_trace[359:356] }, { 2'h0, pc_trace[355:352] } };
         12 : debug_Dout <= { { 2'h0, pc_trace[415:412] }, { 2'h0, pc_trace[411:408] }, { 2'h0, pc_trace[407:404] }, { 2'h0, pc_trace[403:400] },
                              { 2'h0, pc_trace[399:396] }, { 2'h0, pc_trace[395:392] }, { 2'h0, pc_trace[391:388] }, { 2'h0, pc_trace[387:384] } };
         13 : debug_Dout <= { { 2'h0, pc_trace[447:444] }, { 2'h0, pc_trace[443:440] }, { 2'h0, pc_trace[439:436] }, { 2'h0, pc_trace[435:432] },
                              { 2'h0, pc_trace[431:428] }, { 2'h0, pc_trace[427:424] }, { 2'h0, pc_trace[423:420] }, { 2'h0, pc_trace[419:416] } };
         14 : debug_Dout <= { { 2'h0, pc_trace[479:476] }, { 2'h0, pc_trace[475:472] }, { 2'h0, pc_trace[471:468] }, { 2'h0, pc_trace[467:464] },
                              { 2'h0, pc_trace[463:460] }, { 2'h0, pc_trace[459:456] }, { 2'h0, pc_trace[455:452] }, { 2'h0, pc_trace[451:448] } };
         15 : debug_Dout <= { { 2'h0, pc_trace[511:508] }, { 2'h0, pc_trace[507:504] }, { 2'h0, pc_trace[503:500] }, { 2'h0, pc_trace[499:496] },
                              { 2'h0, pc_trace[495:492] }, { 2'h0, pc_trace[491:488] }, { 2'h0, pc_trace[487:484] }, { 2'h0, pc_trace[483:480] } };
         16 : debug_Dout <= { { 2'h0, pc_trace[543:540] }, { 2'h0, pc_trace[539:536] }, { 2'h0, pc_trace[535:532] }, { 2'h0, pc_trace[531:528] },
                              { 2'h0, pc_trace[527:524] }, { 2'h0, pc_trace[523:520] }, { 2'h0, pc_trace[519:516] }, { 2'h0, pc_trace[515:512] } };
         17 : debug_Dout <= { { 2'h0, pc_trace[575:572] }, { 2'h0, pc_trace[571:568] }, { 2'h0, pc_trace[567:564] }, { 2'h0, pc_trace[563:560] },
                              { 2'h0, pc_trace[559:556] }, { 2'h0, pc_trace[555:552] }, { 2'h0, pc_trace[551:548] }, { 2'h0, pc_trace[547:544] } };
         18 : debug_Dout <= { { 2'h0, pc_trace[607:604] }, { 2'h0, pc_trace[603:600] }, { 2'h0, pc_trace[599:596] }, { 2'h0, pc_trace[595:592] },
                              { 2'h0, pc_trace[591:588] }, { 2'h0, pc_trace[587:584] }, { 2'h0, pc_trace[583:580] }, { 2'h0, pc_trace[579:576] } };
         19 : debug_Dout <= { { 2'h0, pc_trace[639:636] }, { 2'h0, pc_trace[635:632] }, { 2'h0, pc_trace[631:628] }, { 2'h0, pc_trace[627:624] },
                              { 2'h0, pc_trace[623:620] }, { 2'h0, pc_trace[619:616] }, { 2'h0, pc_trace[615:612] }, { 2'h0, pc_trace[611:608] } };
         20 : debug_Dout <= { { 2'h0, pc_trace[671:668] }, { 2'h0, pc_trace[667:664] }, { 2'h0, pc_trace[663:660] }, { 2'h0, pc_trace[659:656] },
                              { 2'h0, pc_trace[655:652] }, { 2'h0, pc_trace[651:648] }, { 2'h0, pc_trace[647:644] }, { 2'h0, pc_trace[643:640] } };
         21 : debug_Dout <= { { 2'h0, pc_trace[703:700] }, { 2'h0, pc_trace[699:696] }, { 2'h0, pc_trace[695:692] }, { 2'h0, pc_trace[691:688] },
                              { 2'h0, pc_trace[687:684] }, { 2'h0, pc_trace[683:680] }, { 2'h0, pc_trace[679:676] }, { 2'h0, pc_trace[675:672] } };
         22 : debug_Dout <= { { 2'h0, pc_trace[735:732] }, { 2'h0, pc_trace[731:728] }, { 2'h0, pc_trace[727:724] }, { 2'h0, pc_trace[723:720] },
                              { 2'h0, pc_trace[719:716] }, { 2'h0, pc_trace[715:712] }, { 2'h0, pc_trace[711:708] }, { 2'h0, pc_trace[707:704] } };
         23 : debug_Dout <= { { 2'h0, pc_trace[767:764] }, { 2'h0, pc_trace[763:760] }, { 2'h0, pc_trace[759:756] }, { 2'h0, pc_trace[755:752] },
                              { 2'h0, pc_trace[751:748] }, { 2'h0, pc_trace[747:744] }, { 2'h0, pc_trace[743:740] }, { 2'h0, pc_trace[739:736] } };
         24 : debug_Dout <= { { 2'h0, pc_trace[799:796] }, { 2'h0, pc_trace[795:792] }, { 2'h0, pc_trace[791:788] }, { 2'h0, pc_trace[787:784] },
                              { 2'h0, pc_trace[783:780] }, { 2'h0, pc_trace[779:776] }, { 2'h0, pc_trace[775:772] }, { 2'h0, pc_trace[771:768] } };
         25 : debug_Dout <= { { 2'h0, pc_trace[831:828] }, { 2'h0, pc_trace[827:824] }, { 2'h0, pc_trace[823:820] }, { 2'h0, pc_trace[819:816] },
                              { 2'h0, pc_trace[815:812] }, { 2'h0, pc_trace[811:808] }, { 2'h0, pc_trace[807:804] }, { 2'h0, pc_trace[803:800] } };
         26 : debug_Dout <= { { 2'h0, pc_trace[863:860] }, { 2'h0, pc_trace[859:856] }, { 2'h0, pc_trace[855:852] }, { 2'h0, pc_trace[851:848] },
                              { 2'h0, pc_trace[847:844] }, { 2'h0, pc_trace[843:840] }, { 2'h0, pc_trace[839:836] }, { 2'h0, pc_trace[835:832] } };
         27 : debug_Dout <= { { 2'h0, pc_trace[895:892] }, { 2'h0, pc_trace[891:888] }, { 2'h0, pc_trace[887:884] }, { 2'h0, pc_trace[883:880] },
                              { 2'h0, pc_trace[879:876] }, { 2'h0, pc_trace[875:872] }, { 2'h0, pc_trace[871:868] }, { 2'h0, pc_trace[867:864] } };
         28 : debug_Dout <= { { 2'h0, pc_trace[927:924] }, { 2'h0, pc_trace[923:920] }, { 2'h0, pc_trace[919:916] }, { 2'h0, pc_trace[915:912] },
                              { 2'h0, pc_trace[911:908] }, { 2'h0, pc_trace[907:904] }, { 2'h0, pc_trace[903:900] }, { 2'h0, pc_trace[899:896] } };
         29 : debug_Dout <= { { 2'h0, pc_trace[959:956] }, { 2'h0, pc_trace[955:952] }, { 2'h0, pc_trace[951:948] }, { 2'h0, pc_trace[947:944] },
                              { 2'h0, pc_trace[943:940] }, { 2'h0, pc_trace[939:936] }, { 2'h0, pc_trace[935:932] }, { 2'h0, pc_trace[931:928] } };
    default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
        endcase
      end
   2:
      begin
        case(debug_Address)
            0 : debug_Dout <= { { 2'h0, ad_trace[31:28] },   { 2'h0, ad_trace[27:24] },   { 2'h0, ad_trace[23:20] },   { 2'h0, ad_trace[19:16]   },
                                { 2'h0, ad_trace[15:12] },   { 2'h0, ad_trace[11:8] },    { 2'h0, ad_trace[7:4] },     { 2'h0, ad_trace[3:0]     } };
            1 : debug_Dout <= { { 2'h0, ad_trace[63:60] },   { 2'h0, ad_trace[59:56] },   { 2'h0, ad_trace[55:52] },   { 2'h0, ad_trace[51:48]   },
                                { 2'h0, ad_trace[47:44] },   { 2'h0, ad_trace[43:40] },   { 2'h0, ad_trace[39:36] },   { 2'h0, ad_trace[35:32]   } };
            2 : debug_Dout <= { { 2'h0, ad_trace[95:92] },   { 2'h0, ad_trace[91:88] },   { 2'h0, ad_trace[87:84] },   { 2'h0, ad_trace[83:80]   },
                                { 2'h0, ad_trace[79:76] },   { 2'h0, ad_trace[75:72] },   { 2'h0, ad_trace[71:68] },   { 2'h0, ad_trace[67:64]   } };
            3 : debug_Dout <= { { 2'h0, ad_trace[127:124] }, { 2'h0, ad_trace[123:120] }, { 2'h0, ad_trace[119:116] }, { 2'h0, ad_trace[115:112] },
                                { 2'h0, ad_trace[111:108] }, { 2'h0, ad_trace[107:104] }, { 2'h0, ad_trace[103:100] }, { 2'h0, ad_trace[99:96]   } };
            4 : debug_Dout <= { { 2'h0, ad_trace[159:156] }, { 2'h0, ad_trace[155:152] }, { 2'h0, ad_trace[151:148] }, { 2'h0, ad_trace[147:144] },
                                { 2'h0, ad_trace[143:140] }, { 2'h0, ad_trace[139:136] }, { 2'h0, ad_trace[135:132] }, { 2'h0, ad_trace[131:128] } };
            5 : debug_Dout <= { { 2'h0, ad_trace[191:188] }, { 2'h0, ad_trace[187:184] }, { 2'h0, ad_trace[183:180] }, { 2'h0, ad_trace[179:176] },
                                { 2'h0, ad_trace[175:172] }, { 2'h0, ad_trace[171:168] }, { 2'h0, ad_trace[167:164] }, { 2'h0, ad_trace[163:160] } };
            6 : debug_Dout <= { { 2'h0, ad_trace[223:220] }, { 2'h0, ad_trace[219:216] }, { 2'h0, ad_trace[215:212] }, { 2'h0, ad_trace[211:208] },
                                { 2'h0, ad_trace[207:204] }, { 2'h0, ad_trace[203:200] }, { 2'h0, ad_trace[199:196] }, { 2'h0, ad_trace[195:192] } };
            7 : debug_Dout <= { { 2'h0, ad_trace[255:252] }, { 2'h0, ad_trace[251:248] }, { 2'h0, ad_trace[247:244] }, { 2'h0, ad_trace[243:240] },
                                { 2'h0, ad_trace[239:236] }, { 2'h0, ad_trace[235:232] }, { 2'h0, ad_trace[231:228] }, { 2'h0, ad_trace[227:224] } };
            8 : debug_Dout <= { { 2'h0, ad_trace[287:284] }, { 2'h0, ad_trace[283:280] }, { 2'h0, ad_trace[279:276] }, { 2'h0, ad_trace[275:272] },
                                { 2'h0, ad_trace[271:268] }, { 2'h0, ad_trace[267:264] }, { 2'h0, ad_trace[263:260] }, { 2'h0, ad_trace[259:256] } };
            9 : debug_Dout <= { { 2'h0, ad_trace[319:316] }, { 2'h0, ad_trace[315:312] }, { 2'h0, ad_trace[311:308] }, { 2'h0, ad_trace[307:304] },
                                { 2'h0, ad_trace[303:300] }, { 2'h0, ad_trace[299:296] }, { 2'h0, ad_trace[295:292] }, { 2'h0, ad_trace[291:288] } };
           10 : debug_Dout <= { { 2'h0, ad_trace[351:348] }, { 2'h0, ad_trace[347:344] }, { 2'h0, ad_trace[343:340] }, { 2'h0, ad_trace[339:336] },
                                { 2'h0, ad_trace[335:332] }, { 2'h0, ad_trace[331:328] }, { 2'h0, ad_trace[327:324] }, { 2'h0, ad_trace[323:320] } };
           11 : debug_Dout <= { { 2'h0, ad_trace[383:380] }, { 2'h0, ad_trace[379:376] }, { 2'h0, ad_trace[375:372] }, { 2'h0, ad_trace[371:368] },
                                { 2'h0, ad_trace[367:364] }, { 2'h0, ad_trace[363:360] }, { 2'h0, ad_trace[359:356] }, { 2'h0, ad_trace[355:352] } };
           12 : debug_Dout <= { { 2'h0, ad_trace[415:412] }, { 2'h0, ad_trace[411:408] }, { 2'h0, ad_trace[407:404] }, { 2'h0, ad_trace[403:400] },
                                { 2'h0, ad_trace[399:396] }, { 2'h0, ad_trace[395:392] }, { 2'h0, ad_trace[391:388] }, { 2'h0, ad_trace[387:384] } };
           13 : debug_Dout <= { { 2'h0, ad_trace[447:444] }, { 2'h0, ad_trace[443:440] }, { 2'h0, ad_trace[439:436] }, { 2'h0, ad_trace[435:432] },
                                { 2'h0, ad_trace[431:428] }, { 2'h0, ad_trace[427:424] }, { 2'h0, ad_trace[423:420] }, { 2'h0, ad_trace[419:416] } };
           14 : debug_Dout <= { { 2'h0, ad_trace[479:476] }, { 2'h0, ad_trace[475:472] }, { 2'h0, ad_trace[471:468] }, { 2'h0, ad_trace[467:464] },
                                { 2'h0, ad_trace[463:460] }, { 2'h0, ad_trace[459:456] }, { 2'h0, ad_trace[455:452] }, { 2'h0, ad_trace[451:448] } };
           15 : debug_Dout <= { { 2'h0, ad_trace[511:508] }, { 2'h0, ad_trace[507:504] }, { 2'h0, ad_trace[503:500] }, { 2'h0, ad_trace[499:496] },
                                { 2'h0, ad_trace[495:492] }, { 2'h0, ad_trace[491:488] }, { 2'h0, ad_trace[487:484] }, { 2'h0, ad_trace[483:480] } };
           16 : debug_Dout <= { { 2'h0, ad_trace[543:540] }, { 2'h0, ad_trace[539:536] }, { 2'h0, ad_trace[535:532] }, { 2'h0, ad_trace[531:528] },
                                { 2'h0, ad_trace[527:524] }, { 2'h0, ad_trace[523:520] }, { 2'h0, ad_trace[519:516] }, { 2'h0, ad_trace[515:512] } };
           17 : debug_Dout <= { { 2'h0, ad_trace[575:572] }, { 2'h0, ad_trace[571:568] }, { 2'h0, ad_trace[567:564] }, { 2'h0, ad_trace[563:560] },
                                { 2'h0, ad_trace[559:556] }, { 2'h0, ad_trace[555:552] }, { 2'h0, ad_trace[551:548] }, { 2'h0, ad_trace[547:544] } };
           18 : debug_Dout <= { { 2'h0, ad_trace[607:604] }, { 2'h0, ad_trace[603:600] }, { 2'h0, ad_trace[599:596] }, { 2'h0, ad_trace[595:592] },
                                { 2'h0, ad_trace[591:588] }, { 2'h0, ad_trace[587:584] }, { 2'h0, ad_trace[583:580] }, { 2'h0, ad_trace[579:576] } };
           19 : debug_Dout <= { { 2'h0, ad_trace[639:636] }, { 2'h0, ad_trace[635:632] }, { 2'h0, ad_trace[631:628] }, { 2'h0, ad_trace[627:624] },
                                { 2'h0, ad_trace[623:620] }, { 2'h0, ad_trace[619:616] }, { 2'h0, ad_trace[615:612] }, { 2'h0, ad_trace[611:608] } };
           20 : debug_Dout <= { { 2'h0, ad_trace[671:668] }, { 2'h0, ad_trace[667:664] }, { 2'h0, ad_trace[663:660] }, { 2'h0, ad_trace[659:656] },
                                { 2'h0, ad_trace[655:652] }, { 2'h0, ad_trace[651:648] }, { 2'h0, ad_trace[647:644] }, { 2'h0, ad_trace[643:640] } };
           21 : debug_Dout <= { { 2'h0, ad_trace[703:700] }, { 2'h0, ad_trace[699:696] }, { 2'h0, ad_trace[695:692] }, { 2'h0, ad_trace[691:688] },
                                { 2'h0, ad_trace[687:684] }, { 2'h0, ad_trace[683:680] }, { 2'h0, ad_trace[679:676] }, { 2'h0, ad_trace[675:672] } };
           22 : debug_Dout <= { { 2'h0, ad_trace[735:732] }, { 2'h0, ad_trace[731:728] }, { 2'h0, ad_trace[727:724] }, { 2'h0, ad_trace[723:720] },
                                { 2'h0, ad_trace[719:716] }, { 2'h0, ad_trace[715:712] }, { 2'h0, ad_trace[711:708] }, { 2'h0, ad_trace[707:704] } };
           23 : debug_Dout <= { { 2'h0, ad_trace[767:764] }, { 2'h0, ad_trace[763:760] }, { 2'h0, ad_trace[759:756] }, { 2'h0, ad_trace[755:752] },
                                { 2'h0, ad_trace[751:748] }, { 2'h0, ad_trace[747:744] }, { 2'h0, ad_trace[743:740] }, { 2'h0, ad_trace[739:736] } };
           24 : debug_Dout <= { { 2'h0, ad_trace[799:796] }, { 2'h0, ad_trace[795:792] }, { 2'h0, ad_trace[791:788] }, { 2'h0, ad_trace[787:784] },
                                { 2'h0, ad_trace[783:780] }, { 2'h0, ad_trace[779:776] }, { 2'h0, ad_trace[775:772] }, { 2'h0, ad_trace[771:768] } };
           25 : debug_Dout <= { { 2'h0, ad_trace[831:828] }, { 2'h0, ad_trace[827:824] }, { 2'h0, ad_trace[823:820] }, { 2'h0, ad_trace[819:816] },
                                { 2'h0, ad_trace[815:812] }, { 2'h0, ad_trace[811:808] }, { 2'h0, ad_trace[807:804] }, { 2'h0, ad_trace[803:800] } };
           26 : debug_Dout <= { { 2'h0, ad_trace[863:860] }, { 2'h0, ad_trace[859:856] }, { 2'h0, ad_trace[855:852] }, { 2'h0, ad_trace[851:848] },
                                { 2'h0, ad_trace[847:844] }, { 2'h0, ad_trace[843:840] }, { 2'h0, ad_trace[839:836] }, { 2'h0, ad_trace[835:832] } };
           27 : debug_Dout <= { { 2'h0, ad_trace[895:892] }, { 2'h0, ad_trace[891:888] }, { 2'h0, ad_trace[887:884] }, { 2'h0, ad_trace[883:880] },
                                { 2'h0, ad_trace[879:876] }, { 2'h0, ad_trace[875:872] }, { 2'h0, ad_trace[871:868] }, { 2'h0, ad_trace[867:864] } };
           28 : debug_Dout <= { { 2'h0, ad_trace[927:924] }, { 2'h0, ad_trace[923:920] }, { 2'h0, ad_trace[919:916] }, { 2'h0, ad_trace[915:912] },
                                { 2'h0, ad_trace[911:908] }, { 2'h0, ad_trace[907:904] }, { 2'h0, ad_trace[903:900] }, { 2'h0, ad_trace[899:896] } };
           29 : debug_Dout <= { { 2'h0, ad_trace[959:956] }, { 2'h0, ad_trace[955:952] }, { 2'h0, ad_trace[951:948] }, { 2'h0, ad_trace[947:944] },
                                { 2'h0, ad_trace[943:940] }, { 2'h0, ad_trace[939:936] }, { 2'h0, ad_trace[935:932] }, { 2'h0, ad_trace[931:928] } };

        default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
          endcase
          end
          
   3:
      begin
        case(debug_Address)
            0 : debug_Dout <= { { 2'h0, aw_trace[31:28] },   { 2'h0, aw_trace[27:24] },   { 2'h0, aw_trace[23:20] },   { 2'h0, aw_trace[19:16]   },
                                { 2'h0, aw_trace[15:12] },   { 2'h0, aw_trace[11:8] },    { 2'h0, aw_trace[7:4] },     { 2'h0, aw_trace[3:0]     } };
            1 : debug_Dout <= { { 2'h0, aw_trace[63:60] },   { 2'h0, aw_trace[59:56] },   { 2'h0, aw_trace[55:52] },   { 2'h0, aw_trace[51:48]   },
                                { 2'h0, aw_trace[47:44] },   { 2'h0, aw_trace[43:40] },   { 2'h0, aw_trace[39:36] },   { 2'h0, aw_trace[35:32]   } };
            2 : debug_Dout <= { { 2'h0, aw_trace[95:92] },   { 2'h0, aw_trace[91:88] },   { 2'h0, aw_trace[87:84] },   { 2'h0, aw_trace[83:80]   },
                                { 2'h0, aw_trace[79:76] },   { 2'h0, aw_trace[75:72] },   { 2'h0, aw_trace[71:68] },   { 2'h0, aw_trace[67:64]   } };
            3 : debug_Dout <= { { 2'h0, aw_trace[127:124] }, { 2'h0, aw_trace[123:120] }, { 2'h0, aw_trace[119:116] }, { 2'h0, aw_trace[115:112] },
                                { 2'h0, aw_trace[111:108] }, { 2'h0, aw_trace[107:104] }, { 2'h0, aw_trace[103:100] }, { 2'h0, aw_trace[99:96]   } };
            4 : debug_Dout <= { { 2'h0, aw_trace[159:156] }, { 2'h0, aw_trace[155:152] }, { 2'h0, aw_trace[151:148] }, { 2'h0, aw_trace[147:144] },
                                { 2'h0, aw_trace[143:140] }, { 2'h0, aw_trace[139:136] }, { 2'h0, aw_trace[135:132] }, { 2'h0, aw_trace[131:128] } };
            5 : debug_Dout <= { { 2'h0, aw_trace[191:188] }, { 2'h0, aw_trace[187:184] }, { 2'h0, aw_trace[183:180] }, { 2'h0, aw_trace[179:176] },
                                { 2'h0, aw_trace[175:172] }, { 2'h0, aw_trace[171:168] }, { 2'h0, aw_trace[167:164] }, { 2'h0, aw_trace[163:160] } };
            6 : debug_Dout <= { { 2'h0, aw_trace[223:220] }, { 2'h0, aw_trace[219:216] }, { 2'h0, aw_trace[215:212] }, { 2'h0, aw_trace[211:208] },
                                { 2'h0, aw_trace[207:204] }, { 2'h0, aw_trace[203:200] }, { 2'h0, aw_trace[199:196] }, { 2'h0, aw_trace[195:192] } };
            7 : debug_Dout <= { { 2'h0, aw_trace[255:252] }, { 2'h0, aw_trace[251:248] }, { 2'h0, aw_trace[247:244] }, { 2'h0, aw_trace[243:240] },
                                { 2'h0, aw_trace[239:236] }, { 2'h0, aw_trace[235:232] }, { 2'h0, aw_trace[231:228] }, { 2'h0, aw_trace[227:224] } };
            8 : debug_Dout <= { { 2'h0, aw_trace[287:284] }, { 2'h0, aw_trace[283:280] }, { 2'h0, aw_trace[279:276] }, { 2'h0, aw_trace[275:272] },
                                { 2'h0, aw_trace[271:268] }, { 2'h0, aw_trace[267:264] }, { 2'h0, aw_trace[263:260] }, { 2'h0, aw_trace[259:256] } };
            9 : debug_Dout <= { { 2'h0, aw_trace[319:316] }, { 2'h0, aw_trace[315:312] }, { 2'h0, aw_trace[311:308] }, { 2'h0, aw_trace[307:304] },
                                { 2'h0, aw_trace[303:300] }, { 2'h0, aw_trace[299:296] }, { 2'h0, aw_trace[295:292] }, { 2'h0, aw_trace[291:288] } };
           10 : debug_Dout <= { { 2'h0, aw_trace[351:348] }, { 2'h0, aw_trace[347:344] }, { 2'h0, aw_trace[343:340] }, { 2'h0, aw_trace[339:336] },
                                { 2'h0, aw_trace[335:332] }, { 2'h0, aw_trace[331:328] }, { 2'h0, aw_trace[327:324] }, { 2'h0, aw_trace[323:320] } };
           11 : debug_Dout <= { { 2'h0, aw_trace[383:380] }, { 2'h0, aw_trace[379:376] }, { 2'h0, aw_trace[375:372] }, { 2'h0, aw_trace[371:368] },
                                { 2'h0, aw_trace[367:364] }, { 2'h0, aw_trace[363:360] }, { 2'h0, aw_trace[359:356] }, { 2'h0, aw_trace[355:352] } };
           12 : debug_Dout <= { { 2'h0, aw_trace[415:412] }, { 2'h0, aw_trace[411:408] }, { 2'h0, aw_trace[407:404] }, { 2'h0, aw_trace[403:400] },
                                { 2'h0, aw_trace[399:396] }, { 2'h0, aw_trace[395:392] }, { 2'h0, aw_trace[391:388] }, { 2'h0, aw_trace[387:384] } };
           13 : debug_Dout <= { { 2'h0, aw_trace[447:444] }, { 2'h0, aw_trace[443:440] }, { 2'h0, aw_trace[439:436] }, { 2'h0, aw_trace[435:432] },
                                { 2'h0, aw_trace[431:428] }, { 2'h0, aw_trace[427:424] }, { 2'h0, aw_trace[423:420] }, { 2'h0, aw_trace[419:416] } };
           14 : debug_Dout <= { { 2'h0, aw_trace[479:476] }, { 2'h0, aw_trace[475:472] }, { 2'h0, aw_trace[471:468] }, { 2'h0, aw_trace[467:464] },
                                { 2'h0, aw_trace[463:460] }, { 2'h0, aw_trace[459:456] }, { 2'h0, aw_trace[455:452] }, { 2'h0, aw_trace[451:448] } };
           15 : debug_Dout <= { { 2'h0, aw_trace[511:508] }, { 2'h0, aw_trace[507:504] }, { 2'h0, aw_trace[503:500] }, { 2'h0, aw_trace[499:496] },
                                { 2'h0, aw_trace[495:492] }, { 2'h0, aw_trace[491:488] }, { 2'h0, aw_trace[487:484] }, { 2'h0, aw_trace[483:480] } };
           16 : debug_Dout <= { { 2'h0, aw_trace[543:540] }, { 2'h0, aw_trace[539:536] }, { 2'h0, aw_trace[535:532] }, { 2'h0, aw_trace[531:528] },
                                { 2'h0, aw_trace[527:524] }, { 2'h0, aw_trace[523:520] }, { 2'h0, aw_trace[519:516] }, { 2'h0, aw_trace[515:512] } };
           17 : debug_Dout <= { { 2'h0, aw_trace[575:572] }, { 2'h0, aw_trace[571:568] }, { 2'h0, aw_trace[567:564] }, { 2'h0, aw_trace[563:560] },
                                { 2'h0, aw_trace[559:556] }, { 2'h0, aw_trace[555:552] }, { 2'h0, aw_trace[551:548] }, { 2'h0, aw_trace[547:544] } };
           18 : debug_Dout <= { { 2'h0, aw_trace[607:604] }, { 2'h0, aw_trace[603:600] }, { 2'h0, aw_trace[599:596] }, { 2'h0, aw_trace[595:592] },
                                { 2'h0, aw_trace[591:588] }, { 2'h0, aw_trace[587:584] }, { 2'h0, aw_trace[583:580] }, { 2'h0, aw_trace[579:576] } };
           19 : debug_Dout <= { { 2'h0, aw_trace[639:636] }, { 2'h0, aw_trace[635:632] }, { 2'h0, aw_trace[631:628] }, { 2'h0, aw_trace[627:624] },
                                { 2'h0, aw_trace[623:620] }, { 2'h0, aw_trace[619:616] }, { 2'h0, aw_trace[615:612] }, { 2'h0, aw_trace[611:608] } };
           20 : debug_Dout <= { { 2'h0, aw_trace[671:668] }, { 2'h0, aw_trace[667:664] }, { 2'h0, aw_trace[663:660] }, { 2'h0, aw_trace[659:656] },
                                { 2'h0, aw_trace[655:652] }, { 2'h0, aw_trace[651:648] }, { 2'h0, aw_trace[647:644] }, { 2'h0, aw_trace[643:640] } };
           21 : debug_Dout <= { { 2'h0, aw_trace[703:700] }, { 2'h0, aw_trace[699:696] }, { 2'h0, aw_trace[695:692] }, { 2'h0, aw_trace[691:688] },
                                { 2'h0, aw_trace[687:684] }, { 2'h0, aw_trace[683:680] }, { 2'h0, aw_trace[679:676] }, { 2'h0, aw_trace[675:672] } };
           22 : debug_Dout <= { { 2'h0, aw_trace[735:732] }, { 2'h0, aw_trace[731:728] }, { 2'h0, aw_trace[727:724] }, { 2'h0, aw_trace[723:720] },
                                { 2'h0, aw_trace[719:716] }, { 2'h0, aw_trace[715:712] }, { 2'h0, aw_trace[711:708] }, { 2'h0, aw_trace[707:704] } };
           23 : debug_Dout <= { { 2'h0, aw_trace[767:764] }, { 2'h0, aw_trace[763:760] }, { 2'h0, aw_trace[759:756] }, { 2'h0, aw_trace[755:752] },
                                { 2'h0, aw_trace[751:748] }, { 2'h0, aw_trace[747:744] }, { 2'h0, aw_trace[743:740] }, { 2'h0, aw_trace[739:736] } };
           24 : debug_Dout <= { { 2'h0, aw_trace[799:796] }, { 2'h0, aw_trace[795:792] }, { 2'h0, aw_trace[791:788] }, { 2'h0, aw_trace[787:784] },
                                { 2'h0, aw_trace[783:780] }, { 2'h0, aw_trace[779:776] }, { 2'h0, aw_trace[775:772] }, { 2'h0, aw_trace[771:768] } };
           25 : debug_Dout <= { { 2'h0, aw_trace[831:828] }, { 2'h0, aw_trace[827:824] }, { 2'h0, aw_trace[823:820] }, { 2'h0, aw_trace[819:816] },
                                { 2'h0, aw_trace[815:812] }, { 2'h0, aw_trace[811:808] }, { 2'h0, aw_trace[807:804] }, { 2'h0, aw_trace[803:800] } };
           26 : debug_Dout <= { { 2'h0, aw_trace[863:860] }, { 2'h0, aw_trace[859:856] }, { 2'h0, aw_trace[855:852] }, { 2'h0, aw_trace[851:848] },
                                { 2'h0, aw_trace[847:844] }, { 2'h0, aw_trace[843:840] }, { 2'h0, aw_trace[839:836] }, { 2'h0, aw_trace[835:832] } };
           27 : debug_Dout <= { { 2'h0, aw_trace[895:892] }, { 2'h0, aw_trace[891:888] }, { 2'h0, aw_trace[887:884] }, { 2'h0, aw_trace[883:880] },
                                { 2'h0, aw_trace[879:876] }, { 2'h0, aw_trace[875:872] }, { 2'h0, aw_trace[871:868] }, { 2'h0, aw_trace[867:864] } };
           28 : debug_Dout <= { { 2'h0, aw_trace[927:924] }, { 2'h0, aw_trace[923:920] }, { 2'h0, aw_trace[919:916] }, { 2'h0, aw_trace[915:912] },
                                { 2'h0, aw_trace[911:908] }, { 2'h0, aw_trace[907:904] }, { 2'h0, aw_trace[903:900] }, { 2'h0, aw_trace[899:896] } };
           29 : debug_Dout <= { { 2'h0, aw_trace[959:956] }, { 2'h0, aw_trace[955:952] }, { 2'h0, aw_trace[951:948] }, { 2'h0, aw_trace[947:944] },
                                { 2'h0, aw_trace[943:940] }, { 2'h0, aw_trace[939:936] }, { 2'h0, aw_trace[935:932] }, { 2'h0, aw_trace[931:928] } };

        default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
          endcase
        end

//    5:
//      begin
//        //stack cache
//        case(debug_Address)
//            0 : debug_Dout <= { { 2'h0, cache_dbg0[31:28] },   { 2'h0, cache_dbg0[27:24] },   { 2'h0, cache_dbg0[23:20] },   { 2'h0, cache_dbg0[19:16]   },
//                                { 2'h0, cache_dbg0[15:12] },   { 2'h0, cache_dbg0[11:8] },    { 2'h0, cache_dbg0[7:4] },     { 2'h0, cache_dbg0[3:0]     } };
//            1 : debug_Dout <= { { 2'h0, cache_dbg0[63:60] },   { 2'h0, cache_dbg0[59:56] },   { 2'h0, cache_dbg0[55:52] },   { 2'h0, cache_dbg0[51:48]   },
//                                { 2'h0, cache_dbg0[47:44] },   { 2'h0, cache_dbg0[43:40] },   { 2'h0, cache_dbg0[39:36] },   { 2'h0, cache_dbg0[35:32]   } };
//            2 : debug_Dout <= { { 2'h0, cache_dbg0[95:92] },   { 2'h0, cache_dbg0[91:88] },   { 2'h0, cache_dbg0[87:84] },   { 2'h0, cache_dbg0[83:80]   },
//                                { 2'h0, cache_dbg0[79:76] },   { 2'h0, cache_dbg0[75:72] },   { 2'h0, cache_dbg0[71:68] },   { 2'h0, cache_dbg0[67:64]   } };
//            3 : debug_Dout <= { { 2'h0, cache_dbg0[127:124] }, { 2'h0, cache_dbg0[123:120] }, { 2'h0, cache_dbg0[119:116] }, { 2'h0, cache_dbg0[115:112] },
//                                { 2'h0, cache_dbg0[111:108] }, { 2'h0, cache_dbg0[107:104] }, { 2'h0, cache_dbg0[103:100] }, { 2'h0, cache_dbg0[99:96]   } };
//            4 : debug_Dout <= { { 2'h0, cache_dbg0[159:156] }, { 2'h0, cache_dbg0[155:152] }, { 2'h0, cache_dbg0[151:148] }, { 2'h0, cache_dbg0[147:144] },
//                                { 2'h0, cache_dbg0[143:140] }, { 2'h0, cache_dbg0[139:136] }, { 2'h0, cache_dbg0[135:132] }, { 2'h0, cache_dbg0[131:128] } };
//            5 : debug_Dout <= { { 2'h0, cache_dbg0[191:188] }, { 2'h0, cache_dbg0[187:184] }, { 2'h0, cache_dbg0[183:180] }, { 2'h0, cache_dbg0[179:176] },
//                                { 2'h0, cache_dbg0[175:172] }, { 2'h0, cache_dbg0[171:168] }, { 2'h0, cache_dbg0[167:164] }, { 2'h0, cache_dbg0[163:160] } };
//            6 : debug_Dout <= { { 2'h0, cache_dbg0[223:220] }, { 2'h0, cache_dbg0[219:216] }, { 2'h0, cache_dbg0[215:212] }, { 2'h0, cache_dbg0[211:208] },
//                                { 2'h0, cache_dbg0[207:204] }, { 2'h0, cache_dbg0[203:200] }, { 2'h0, cache_dbg0[199:196] }, { 2'h0, cache_dbg0[195:192] } };
//            7 : debug_Dout <= { { 2'h0, cache_dbg0[255:252] }, { 2'h0, cache_dbg0[251:248] }, { 2'h0, cache_dbg0[247:244] }, { 2'h0, cache_dbg0[243:240] },
//                                { 2'h0, cache_dbg0[239:236] }, { 2'h0, cache_dbg0[235:232] }, { 2'h0, cache_dbg0[231:228] }, { 2'h0, cache_dbg0[227:224] } };
//            8 : debug_Dout <= { { 2'h0, cache_dbg0[287:284] }, { 2'h0, cache_dbg0[283:280] }, { 2'h0, cache_dbg0[279:276] }, { 2'h0, cache_dbg0[275:272] },
//                                { 2'h0, cache_dbg0[271:268] }, { 2'h0, cache_dbg0[267:264] }, { 2'h0, cache_dbg0[263:260] }, { 2'h0, cache_dbg0[259:256] } };
//            9 : debug_Dout <= { { 2'h0, cache_dbg0[319:316] }, { 2'h0, cache_dbg0[315:312] }, { 2'h0, cache_dbg0[311:308] }, { 2'h0, cache_dbg0[307:304] },
//                                { 2'h0, cache_dbg0[303:300] }, { 2'h0, cache_dbg0[299:296] }, { 2'h0, cache_dbg0[295:292] }, { 2'h0, cache_dbg0[291:288] } };
//           10 : debug_Dout <= { { 2'h0, cache_dbg0[351:348] }, { 2'h0, cache_dbg0[347:344] }, { 2'h0, cache_dbg0[343:340] }, { 2'h0, cache_dbg0[339:336] },
//                                { 2'h0, cache_dbg0[335:332] }, { 2'h0, cache_dbg0[331:328] }, { 2'h0, cache_dbg0[327:324] }, { 2'h0, cache_dbg0[323:320] } };
//           11 : debug_Dout <= { { 2'h0, cache_dbg0[383:380] }, { 2'h0, cache_dbg0[379:376] }, { 2'h0, cache_dbg0[375:372] }, { 2'h0, cache_dbg0[371:368] },
//                                { 2'h0, cache_dbg0[367:364] }, { 2'h0, cache_dbg0[363:360] }, { 2'h0, cache_dbg0[359:356] }, { 2'h0, cache_dbg0[355:352] } };
//           12 : debug_Dout <= { { 2'h0, cache_dbg0[415:412] }, { 2'h0, cache_dbg0[411:408] }, { 2'h0, cache_dbg0[407:404] }, { 2'h0, cache_dbg0[403:400] },
//                                { 2'h0, cache_dbg0[399:396] }, { 2'h0, cache_dbg0[395:392] }, { 2'h0, cache_dbg0[391:388] }, { 2'h0, cache_dbg0[387:384] } };
//           13 : debug_Dout <= { { 2'h0, cache_dbg0[447:444] }, { 2'h0, cache_dbg0[443:440] }, { 2'h0, cache_dbg0[439:436] }, { 2'h0, cache_dbg0[435:432] },
//                                { 2'h0, cache_dbg0[431:428] }, { 2'h0, cache_dbg0[427:424] }, { 2'h0, cache_dbg0[423:420] }, { 2'h0, cache_dbg0[419:416] } };
//           14 : debug_Dout <= { { 2'h0, cache_dbg0[479:476] }, { 2'h0, cache_dbg0[475:472] }, { 2'h0, cache_dbg0[471:468] }, { 2'h0, cache_dbg0[467:464] },
//                                { 2'h0, cache_dbg0[463:460] }, { 2'h0, cache_dbg0[459:456] }, { 2'h0, cache_dbg0[455:452] }, { 2'h0, cache_dbg0[451:448] } };
//           15 : debug_Dout <= { { 2'h0, cache_dbg0[511:508] }, { 2'h0, cache_dbg0[507:504] }, { 2'h0, cache_dbg0[503:500] }, { 2'h0, cache_dbg0[499:496] },
//                                { 2'h0, cache_dbg0[495:492] }, { 2'h0, cache_dbg0[491:488] }, { 2'h0, cache_dbg0[487:484] }, { 2'h0, cache_dbg0[483:480] } };
//           16 : debug_Dout <= { { 2'h0, cache_dbg0[543:540] }, { 2'h0, cache_dbg0[539:536] }, { 2'h0, cache_dbg0[535:532] }, { 2'h0, cache_dbg0[531:528] },
//                                { 2'h0, cache_dbg0[527:524] }, { 2'h0, cache_dbg0[523:520] }, { 2'h0, cache_dbg0[519:516] }, { 2'h0, cache_dbg0[515:512] } };
//           17 : debug_Dout <= { { 2'h0, cache_dbg0[575:572] }, { 2'h0, cache_dbg0[571:568] }, { 2'h0, cache_dbg0[567:564] }, { 2'h0, cache_dbg0[563:560] },
//                                { 2'h0, cache_dbg0[559:556] }, { 2'h0, cache_dbg0[555:552] }, { 2'h0, cache_dbg0[551:548] }, { 2'h0, cache_dbg0[547:544] } };
//           18 : debug_Dout <= { { 2'h0, cache_dbg0[607:604] }, { 2'h0, cache_dbg0[603:600] }, { 2'h0, cache_dbg0[599:596] }, { 2'h0, cache_dbg0[595:592] },
//                                { 2'h0, cache_dbg0[591:588] }, { 2'h0, cache_dbg0[587:584] }, { 2'h0, cache_dbg0[583:580] }, { 2'h0, cache_dbg0[579:576] } };
//           19 : debug_Dout <= { { 2'h0, cache_dbg0[639:636] }, { 2'h0, cache_dbg0[635:632] }, { 2'h0, cache_dbg0[631:628] }, { 2'h0, cache_dbg0[627:624] },
//                                { 2'h0, cache_dbg0[623:620] }, { 2'h0, cache_dbg0[619:616] }, { 2'h0, cache_dbg0[615:612] }, { 2'h0, cache_dbg0[611:608] } };
//           20 : debug_Dout <= { { 2'h0, cache_dbg0[671:668] }, { 2'h0, cache_dbg0[667:664] }, { 2'h0, cache_dbg0[663:660] }, { 2'h0, cache_dbg0[659:656] },
//                                { 2'h0, cache_dbg0[655:652] }, { 2'h0, cache_dbg0[651:648] }, { 2'h0, cache_dbg0[647:644] }, { 2'h0, cache_dbg0[643:640] } };
//           21 : debug_Dout <= { { 2'h0, cache_dbg0[703:700] }, { 2'h0, cache_dbg0[699:696] }, { 2'h0, cache_dbg0[695:692] }, { 2'h0, cache_dbg0[691:688] },
//                                { 2'h0, cache_dbg0[687:684] }, { 2'h0, cache_dbg0[683:680] }, { 2'h0, cache_dbg0[679:676] }, { 2'h0, cache_dbg0[675:672] } };
//           22 : debug_Dout <= { { 2'h0, cache_dbg0[735:732] }, { 2'h0, cache_dbg0[731:728] }, { 2'h0, cache_dbg0[727:724] }, { 2'h0, cache_dbg0[723:720] },
//                                { 2'h0, cache_dbg0[719:716] }, { 2'h0, cache_dbg0[715:712] }, { 2'h0, cache_dbg0[711:708] }, { 2'h0, cache_dbg0[707:704] } };
//           23 : debug_Dout <= { { 2'h0, cache_dbg0[767:764] }, { 2'h0, cache_dbg0[763:760] }, { 2'h0, cache_dbg0[759:756] }, { 2'h0, cache_dbg0[755:752] },
//                                { 2'h0, cache_dbg0[751:748] }, { 2'h0, cache_dbg0[747:744] }, { 2'h0, cache_dbg0[743:740] }, { 2'h0, cache_dbg0[739:736] } };
//           24 : debug_Dout <= { { 2'h0, cache_dbg0[799:796] }, { 2'h0, cache_dbg0[795:792] }, { 2'h0, cache_dbg0[791:788] }, { 2'h0, cache_dbg0[787:784] },
//                                { 2'h0, cache_dbg0[783:780] }, { 2'h0, cache_dbg0[779:776] }, { 2'h0, cache_dbg0[775:772] }, { 2'h0, cache_dbg0[771:768] } };
//           25 : debug_Dout <= { { 2'h0, cache_dbg0[831:828] }, { 2'h0, cache_dbg0[827:824] }, { 2'h0, cache_dbg0[823:820] }, { 2'h0, cache_dbg0[819:816] },
//                                { 2'h0, cache_dbg0[815:812] }, { 2'h0, cache_dbg0[811:808] }, { 2'h0, cache_dbg0[807:804] }, { 2'h0, cache_dbg0[803:800] } };
//           26 : debug_Dout <= { { 2'h0, cache_dbg0[863:860] }, { 2'h0, cache_dbg0[859:856] }, { 2'h0, cache_dbg0[855:852] }, { 2'h0, cache_dbg0[851:848] },
//                                { 2'h0, cache_dbg0[847:844] }, { 2'h0, cache_dbg0[843:840] }, { 2'h0, cache_dbg0[839:836] }, { 2'h0, cache_dbg0[835:832] } };
//           27 : debug_Dout <= { { 2'h0, cache_dbg0[895:892] }, { 2'h0, cache_dbg0[891:888] }, { 2'h0, cache_dbg0[887:884] }, { 2'h0, cache_dbg0[883:880] },
//                                { 2'h0, cache_dbg0[879:876] }, { 2'h0, cache_dbg0[875:872] }, { 2'h0, cache_dbg0[871:868] }, { 2'h0, cache_dbg0[867:864] } };
//           28 : debug_Dout <= { { 2'h0, cache_dbg0[927:924] }, { 2'h0, cache_dbg0[923:920] }, { 2'h0, cache_dbg0[919:916] }, { 2'h0, cache_dbg0[915:912] },
//                                { 2'h0, cache_dbg0[911:908] }, { 2'h0, cache_dbg0[907:904] }, { 2'h0, cache_dbg0[903:900] }, { 2'h0, cache_dbg0[899:896] } };
//           29 : debug_Dout <= { { 2'h0, cache_dbg0[959:956] }, { 2'h0, cache_dbg0[955:952] }, { 2'h0, cache_dbg0[951:948] }, { 2'h0, cache_dbg0[947:944] },
//                                { 2'h0, cache_dbg0[943:940] }, { 2'h0, cache_dbg0[939:936] }, { 2'h0, cache_dbg0[935:932] }, { 2'h0, cache_dbg0[931:928] } };
//
//        default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
//          endcase
//        end
//
//     6:
//      begin
//        //stack cache
//        case(debug_Address)
//            0 : debug_Dout <= { { 2'h0, cache_dbg1[31:28] },   { 2'h0, cache_dbg1[27:24] },   { 2'h0, cache_dbg1[23:20] },   { 2'h0, cache_dbg1[19:16]   },
//                                { 2'h0, cache_dbg1[15:12] },   { 2'h0, cache_dbg1[11:8] },    { 2'h0, cache_dbg1[7:4] },     { 2'h0, cache_dbg1[3:0]     } };
//            1 : debug_Dout <= { { 2'h0, cache_dbg1[63:60] },   { 2'h0, cache_dbg1[59:56] },   { 2'h0, cache_dbg1[55:52] },   { 2'h0, cache_dbg1[51:48]   },
//                                { 2'h0, cache_dbg1[47:44] },   { 2'h0, cache_dbg1[43:40] },   { 2'h0, cache_dbg1[39:36] },   { 2'h0, cache_dbg1[35:32]   } };
//            2 : debug_Dout <= { { 2'h0, cache_dbg1[95:92] },   { 2'h0, cache_dbg1[91:88] },   { 2'h0, cache_dbg1[87:84] },   { 2'h0, cache_dbg1[83:80]   },
//                                { 2'h0, cache_dbg1[79:76] },   { 2'h0, cache_dbg1[75:72] },   { 2'h0, cache_dbg1[71:68] },   { 2'h0, cache_dbg1[67:64]   } };
//            3 : debug_Dout <= { { 2'h0, cache_dbg1[127:124] }, { 2'h0, cache_dbg1[123:120] }, { 2'h0, cache_dbg1[119:116] }, { 2'h0, cache_dbg1[115:112] },
//                                { 2'h0, cache_dbg1[111:108] }, { 2'h0, cache_dbg1[107:104] }, { 2'h0, cache_dbg1[103:100] }, { 2'h0, cache_dbg1[99:96]   } };
//            4 : debug_Dout <= { { 2'h0, cache_dbg1[159:156] }, { 2'h0, cache_dbg1[155:152] }, { 2'h0, cache_dbg1[151:148] }, { 2'h0, cache_dbg1[147:144] },
//                                { 2'h0, cache_dbg1[143:140] }, { 2'h0, cache_dbg1[139:136] }, { 2'h0, cache_dbg1[135:132] }, { 2'h0, cache_dbg1[131:128] } };
//            5 : debug_Dout <= { { 2'h0, cache_dbg1[191:188] }, { 2'h0, cache_dbg1[187:184] }, { 2'h0, cache_dbg1[183:180] }, { 2'h0, cache_dbg1[179:176] },
//                                { 2'h0, cache_dbg1[175:172] }, { 2'h0, cache_dbg1[171:168] }, { 2'h0, cache_dbg1[167:164] }, { 2'h0, cache_dbg1[163:160] } };
//            6 : debug_Dout <= { { 2'h0, cache_dbg1[223:220] }, { 2'h0, cache_dbg1[219:216] }, { 2'h0, cache_dbg1[215:212] }, { 2'h0, cache_dbg1[211:208] },
//                                { 2'h0, cache_dbg1[207:204] }, { 2'h0, cache_dbg1[203:200] }, { 2'h0, cache_dbg1[199:196] }, { 2'h0, cache_dbg1[195:192] } };
//            7 : debug_Dout <= { { 2'h0, cache_dbg1[255:252] }, { 2'h0, cache_dbg1[251:248] }, { 2'h0, cache_dbg1[247:244] }, { 2'h0, cache_dbg1[243:240] },
//                                { 2'h0, cache_dbg1[239:236] }, { 2'h0, cache_dbg1[235:232] }, { 2'h0, cache_dbg1[231:228] }, { 2'h0, cache_dbg1[227:224] } };
//            8 : debug_Dout <= { { 2'h0, cache_dbg1[287:284] }, { 2'h0, cache_dbg1[283:280] }, { 2'h0, cache_dbg1[279:276] }, { 2'h0, cache_dbg1[275:272] },
//                                { 2'h0, cache_dbg1[271:268] }, { 2'h0, cache_dbg1[267:264] }, { 2'h0, cache_dbg1[263:260] }, { 2'h0, cache_dbg1[259:256] } };
//            9 : debug_Dout <= { { 2'h0, cache_dbg1[319:316] }, { 2'h0, cache_dbg1[315:312] }, { 2'h0, cache_dbg1[311:308] }, { 2'h0, cache_dbg1[307:304] },
//                                { 2'h0, cache_dbg1[303:300] }, { 2'h0, cache_dbg1[299:296] }, { 2'h0, cache_dbg1[295:292] }, { 2'h0, cache_dbg1[291:288] } };
//           10 : debug_Dout <= { { 2'h0, cache_dbg1[351:348] }, { 2'h0, cache_dbg1[347:344] }, { 2'h0, cache_dbg1[343:340] }, { 2'h0, cache_dbg1[339:336] },
//                                { 2'h0, cache_dbg1[335:332] }, { 2'h0, cache_dbg1[331:328] }, { 2'h0, cache_dbg1[327:324] }, { 2'h0, cache_dbg1[323:320] } };
//           11 : debug_Dout <= { { 2'h0, cache_dbg1[383:380] }, { 2'h0, cache_dbg1[379:376] }, { 2'h0, cache_dbg1[375:372] }, { 2'h0, cache_dbg1[371:368] },
//                                { 2'h0, cache_dbg1[367:364] }, { 2'h0, cache_dbg1[363:360] }, { 2'h0, cache_dbg1[359:356] }, { 2'h0, cache_dbg1[355:352] } };
//           12 : debug_Dout <= { { 2'h0, cache_dbg1[415:412] }, { 2'h0, cache_dbg1[411:408] }, { 2'h0, cache_dbg1[407:404] }, { 2'h0, cache_dbg1[403:400] },
//                                { 2'h0, cache_dbg1[399:396] }, { 2'h0, cache_dbg1[395:392] }, { 2'h0, cache_dbg1[391:388] }, { 2'h0, cache_dbg1[387:384] } };
//           13 : debug_Dout <= { { 2'h0, cache_dbg1[447:444] }, { 2'h0, cache_dbg1[443:440] }, { 2'h0, cache_dbg1[439:436] }, { 2'h0, cache_dbg1[435:432] },
//                                { 2'h0, cache_dbg1[431:428] }, { 2'h0, cache_dbg1[427:424] }, { 2'h0, cache_dbg1[423:420] }, { 2'h0, cache_dbg1[419:416] } };
//           14 : debug_Dout <= { { 2'h0, cache_dbg1[479:476] }, { 2'h0, cache_dbg1[475:472] }, { 2'h0, cache_dbg1[471:468] }, { 2'h0, cache_dbg1[467:464] },
//                                { 2'h0, cache_dbg1[463:460] }, { 2'h0, cache_dbg1[459:456] }, { 2'h0, cache_dbg1[455:452] }, { 2'h0, cache_dbg1[451:448] } };
//           15 : debug_Dout <= { { 2'h0, cache_dbg1[511:508] }, { 2'h0, cache_dbg1[507:504] }, { 2'h0, cache_dbg1[503:500] }, { 2'h0, cache_dbg1[499:496] },
//                                { 2'h0, cache_dbg1[495:492] }, { 2'h0, cache_dbg1[491:488] }, { 2'h0, cache_dbg1[487:484] }, { 2'h0, cache_dbg1[483:480] } };
//           16 : debug_Dout <= { { 2'h0, cache_dbg1[543:540] }, { 2'h0, cache_dbg1[539:536] }, { 2'h0, cache_dbg1[535:532] }, { 2'h0, cache_dbg1[531:528] },
//                                { 2'h0, cache_dbg1[527:524] }, { 2'h0, cache_dbg1[523:520] }, { 2'h0, cache_dbg1[519:516] }, { 2'h0, cache_dbg1[515:512] } };
//           17 : debug_Dout <= { { 2'h0, cache_dbg1[575:572] }, { 2'h0, cache_dbg1[571:568] }, { 2'h0, cache_dbg1[567:564] }, { 2'h0, cache_dbg1[563:560] },
//                                { 2'h0, cache_dbg1[559:556] }, { 2'h0, cache_dbg1[555:552] }, { 2'h0, cache_dbg1[551:548] }, { 2'h0, cache_dbg1[547:544] } };
//           18 : debug_Dout <= { { 2'h0, cache_dbg1[607:604] }, { 2'h0, cache_dbg1[603:600] }, { 2'h0, cache_dbg1[599:596] }, { 2'h0, cache_dbg1[595:592] },
//                                { 2'h0, cache_dbg1[591:588] }, { 2'h0, cache_dbg1[587:584] }, { 2'h0, cache_dbg1[583:580] }, { 2'h0, cache_dbg1[579:576] } };
//           19 : debug_Dout <= { { 2'h0, cache_dbg1[639:636] }, { 2'h0, cache_dbg1[635:632] }, { 2'h0, cache_dbg1[631:628] }, { 2'h0, cache_dbg1[627:624] },
//                                { 2'h0, cache_dbg1[623:620] }, { 2'h0, cache_dbg1[619:616] }, { 2'h0, cache_dbg1[615:612] }, { 2'h0, cache_dbg1[611:608] } };
//           20 : debug_Dout <= { { 2'h0, cache_dbg1[671:668] }, { 2'h0, cache_dbg1[667:664] }, { 2'h0, cache_dbg1[663:660] }, { 2'h0, cache_dbg1[659:656] },
//                                { 2'h0, cache_dbg1[655:652] }, { 2'h0, cache_dbg1[651:648] }, { 2'h0, cache_dbg1[647:644] }, { 2'h0, cache_dbg1[643:640] } };
//           21 : debug_Dout <= { { 2'h0, cache_dbg1[703:700] }, { 2'h0, cache_dbg1[699:696] }, { 2'h0, cache_dbg1[695:692] }, { 2'h0, cache_dbg1[691:688] },
//                                { 2'h0, cache_dbg1[687:684] }, { 2'h0, cache_dbg1[683:680] }, { 2'h0, cache_dbg1[679:676] }, { 2'h0, cache_dbg1[675:672] } };
//           22 : debug_Dout <= { { 2'h0, cache_dbg1[735:732] }, { 2'h0, cache_dbg1[731:728] }, { 2'h0, cache_dbg1[727:724] }, { 2'h0, cache_dbg1[723:720] },
//                                { 2'h0, cache_dbg1[719:716] }, { 2'h0, cache_dbg1[715:712] }, { 2'h0, cache_dbg1[711:708] }, { 2'h0, cache_dbg1[707:704] } };
//           23 : debug_Dout <= { { 2'h0, cache_dbg1[767:764] }, { 2'h0, cache_dbg1[763:760] }, { 2'h0, cache_dbg1[759:756] }, { 2'h0, cache_dbg1[755:752] },
//                                { 2'h0, cache_dbg1[751:748] }, { 2'h0, cache_dbg1[747:744] }, { 2'h0, cache_dbg1[743:740] }, { 2'h0, cache_dbg1[739:736] } };
//           24 : debug_Dout <= { { 2'h0, cache_dbg1[799:796] }, { 2'h0, cache_dbg1[795:792] }, { 2'h0, cache_dbg1[791:788] }, { 2'h0, cache_dbg1[787:784] },
//                                { 2'h0, cache_dbg1[783:780] }, { 2'h0, cache_dbg1[779:776] }, { 2'h0, cache_dbg1[775:772] }, { 2'h0, cache_dbg1[771:768] } };
//           25 : debug_Dout <= { { 2'h0, cache_dbg1[831:828] }, { 2'h0, cache_dbg1[827:824] }, { 2'h0, cache_dbg1[823:820] }, { 2'h0, cache_dbg1[819:816] },
//                                { 2'h0, cache_dbg1[815:812] }, { 2'h0, cache_dbg1[811:808] }, { 2'h0, cache_dbg1[807:804] }, { 2'h0, cache_dbg1[803:800] } };
//           26 : debug_Dout <= { { 2'h0, cache_dbg1[863:860] }, { 2'h0, cache_dbg1[859:856] }, { 2'h0, cache_dbg1[855:852] }, { 2'h0, cache_dbg1[851:848] },
//                                { 2'h0, cache_dbg1[847:844] }, { 2'h0, cache_dbg1[843:840] }, { 2'h0, cache_dbg1[839:836] }, { 2'h0, cache_dbg1[835:832] } };
//           27 : debug_Dout <= { { 2'h0, cache_dbg1[895:892] }, { 2'h0, cache_dbg1[891:888] }, { 2'h0, cache_dbg1[887:884] }, { 2'h0, cache_dbg1[883:880] },
//                                { 2'h0, cache_dbg1[879:876] }, { 2'h0, cache_dbg1[875:872] }, { 2'h0, cache_dbg1[871:868] }, { 2'h0, cache_dbg1[867:864] } };
//           28 : debug_Dout <= { { 2'h0, cache_dbg1[927:924] }, { 2'h0, cache_dbg1[923:920] }, { 2'h0, cache_dbg1[919:916] }, { 2'h0, cache_dbg1[915:912] },
//                                { 2'h0, cache_dbg1[911:908] }, { 2'h0, cache_dbg1[907:904] }, { 2'h0, cache_dbg1[903:900] }, { 2'h0, cache_dbg1[899:896] } };
//           29 : debug_Dout <= { { 2'h0, cache_dbg1[959:956] }, { 2'h0, cache_dbg1[955:952] }, { 2'h0, cache_dbg1[951:948] }, { 2'h0, cache_dbg1[947:944] },
//                                { 2'h0, cache_dbg1[943:940] }, { 2'h0, cache_dbg1[939:936] }, { 2'h0, cache_dbg1[935:932] }, { 2'h0, cache_dbg1[931:928] } };
//
//        default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
//          endcase
//        end

      default : debug_Dout <= { spc, spc, spc, spc, spc, spc, spc, spc };
    endcase
  end


endmodule


