//****************************************************************************
// C64 glue logic for asynchonous memory architecture system.
//
// (C) 2013, Ken Chamberlain <kenneth.r.chamberlain@comcast.net>
//
// Feel free to use this code in any project (commercial or not), as long as you
// keep this message, and the copyright notice. This code is provided "as is", 
// without any warranties of any kind. 
// 
//****************************************************************************

//`define mouser

module plaA(mclk,
            cpu_DRw, cpu_PR,
            cpu_aw, cpu_awl,
            dAB,
            ram, krnl, basc, char,
            DBRvic, DBRsid, DBRcia1, DBRcia2, DBRio1, DBRio2, DBRcolor,
            vic_A, vic_charRom, vic_bitmap,
            dma_ack, dma_dr, dma_ack3, dma_GAME, dma_EXROM, dma_hiram, dma_loram, dma_charen,
            cpu_GAME, cpu_EXROM, cpu_hiram, cpu_loram, cpu_charen, cpu_rw,
            vic_w_reg, sid_w_reg, cia1_w_reg, cia2_w_reg, io1_w_reg, io2_w_reg,
            vic_r_reg, cia1_r_reg, cia2_r_reg, io1_r_reg, io2_r_reg,
            color_ram_w, ramselw, pRead
           );

input mclk;
input [31:0] ram, krnl, basc, char;
input [15:8] cpu_aw;
input [5:0] cpu_awl;
input [7:0] DBRvic, DBRsid, DBRcia1, DBRcia2, DBRio1, DBRio2;
input [3:0] DBRcolor;
input cpu_GAME, cpu_EXROM, cpu_hiram, cpu_loram, cpu_charen, cpu_rw;
input dma_ack, dma_GAME, dma_EXROM, dma_hiram, dma_loram, dma_charen;
input vic_A, vic_charRom;

//decoded address bus (for periph registers)
`ifdef mouser
output [47:0] dAB;
`else
output [46:0] dAB; 
`endif
output [31:0] cpu_DRw, vic_bitmap;
output [7:0] cpu_PR;
output [7:0] dma_dr;
output dma_ack3;
output vic_w_reg, sid_w_reg, cia1_w_reg, cia2_w_reg, io1_w_reg, io2_w_reg;
output vic_r_reg, cia1_r_reg, cia2_r_reg, io1_r_reg, io2_r_reg;
output color_ram_w, ramselw, pRead;

// parameter definitions
parameter H = 1'b1;
parameter L = 1'b0;

`ifdef mouser
wire [47:0] dAB;
wire [47:0] dAB1;
`else
wire [46:0] dAB;
wire [46:0] dAB1;
`endif
wire [31:0] cpu_DR, cpu_DRw;

reg [7:0] dma_dr, dma_dr2;
reg dma_ack2, dma_ack3;
wire vic_w_reg, sid_w_reg, cia1_w_reg, cia2_w_reg, io1_w_reg, io2_w_reg;
wire vic_r_reg, sid_r_reg, cia1_r_reg, cia2_r_reg, io1_r_reg, io2_r_reg;
wire pRead;

//#################################################
// vic data reads
reg [31:0] vic_bitmap;

//#################################################
// decode of the lower address bits for io regeisters
register_decoder registerw(cpu_awl[5:0], dAB1);

//#################################################
// memory map control
memmap cpu(cpu_GAME, cpu_EXROM, cpu_hiram, cpu_loram, cpu_charen, cpu_IOSpace, cpu_Rom_8, cpu_Rom_A, cpu_Rom_D, cpu_Rom_E);
memmap dma(dma_GAME, dma_EXROM, dma_hiram, dma_loram, dma_charen, dma_IOSpace, dma_Rom_8, dma_Rom_A, dma_Rom_D, dma_Rom_E);

//#################################################          
//select enables for the current bus master
assign IOSpace_w = (~dma_ack) ? dma_IOSpace : cpu_IOSpace;
assign Rom_8 = (~dma_ack) ? dma_Rom_8 : cpu_Rom_8;
assign Rom_A = (~dma_ack) ? dma_Rom_A : cpu_Rom_A;
assign Rom_D = (~dma_ack) ? dma_Rom_D : cpu_Rom_D;
assign Rom_E = (~dma_ack) ? dma_Rom_E : cpu_Rom_E;

//#################################################          
//figure out if io is being accessed
assign io_spacew = IOSpace_w & cpu_aw[15] & cpu_aw[14] & ~cpu_aw[13] & cpu_aw[12];

//#################################################          
//acessing stuff in ram or io
// a[17:16]
// 0x00 ram
// 0x01 rom
// 0x10 io image (includes color ram)
// 0x11 hi-res graphics, 8 bit color per pixel ??? - not implemented

//A16
assign ramw = (~vic_A) ? ~vic_charRom : (Rom_8 & cpu_aw[15] & ~cpu_aw[14] & ~cpu_aw[13]) |
                                        (Rom_A & cpu_aw[15] & ~cpu_aw[14] &  cpu_aw[13]) |
                                        (Rom_D & cpu_aw[15] &  cpu_aw[14] & ~cpu_aw[13] & cpu_aw[12]) |
                                        (Rom_E & cpu_aw[15] &  cpu_aw[14] &  cpu_aw[13]);

//A17
assign iow = (~vic_A) ? L : io_spacew;

//#################################################          
//ram/rom selects
assign basicenw  = Rom_A & ramw & ~iow & cpu_aw[15] & ~cpu_aw[14] & cpu_aw[13];
assign kernelenw = Rom_E & ramw & ~iow & cpu_aw[15] & cpu_aw[14] & cpu_aw[13];
assign charenw   = ~vic_A ? ~vic_charRom : Rom_D & ramw & ~iow & cpu_aw[15] & cpu_aw[14] & ~cpu_aw[13] & cpu_aw[12];
assign ramselw   = ~iow & ~ramw | (~cpu_rw & (basicenw | kernelenw | charenw)); //write to ram even if roms enabled

//#################################################          
//generate the peripheral read and write strobe signals

assign vicr   = io_spacew & cpu_rw & ~cpu_aw[11] & ~cpu_aw[10];
assign sidr   = io_spacew & cpu_rw & ~cpu_aw[11] &  cpu_aw[10];
assign cia1r  = io_spacew & cpu_rw &  cpu_aw[11] &  cpu_aw[10] & ~cpu_aw[9] & ~cpu_aw[8];
assign cia2r  = io_spacew & cpu_rw &  cpu_aw[11] &  cpu_aw[10] & ~cpu_aw[9] &  cpu_aw[8];
assign io1r   = io_spacew & cpu_rw &  cpu_aw[11] &  cpu_aw[10] &  cpu_aw[9] & ~cpu_aw[8];
assign io2r   = io_spacew & cpu_rw &  cpu_aw[11] &  cpu_aw[10] &  cpu_aw[9] &  cpu_aw[8];
assign colorr = io_spacew & cpu_rw &  cpu_aw[11] & ~cpu_aw[10];

assign vicw   = io_spacew & ~cpu_rw & ~cpu_aw[11] & ~cpu_aw[10];
assign sidw   = io_spacew & ~cpu_rw & ~cpu_aw[11] &  cpu_aw[10];
assign cia1w  = io_spacew & ~cpu_rw &  cpu_aw[11] &  cpu_aw[10] & ~cpu_aw[9] & ~cpu_aw[8];
assign cia2w  = io_spacew & ~cpu_rw &  cpu_aw[11] &  cpu_aw[10] & ~cpu_aw[9] &  cpu_aw[8];
assign io1w   = io_spacew & ~cpu_rw &  cpu_aw[11] &  cpu_aw[10] &  cpu_aw[9] & ~cpu_aw[8];
assign io2w   = io_spacew & ~cpu_rw &  cpu_aw[11] &  cpu_aw[10] &  cpu_aw[9] &  cpu_aw[8];
assign colorw = io_spacew & ~cpu_rw &  cpu_aw[11] & ~cpu_aw[10];

//this is used asynchronously because it accesses a bram
assign color_ram_w = ~colorw;

//this is used asynchronously to allow more processing to occur before the clock cycle ends
assign cpu_DRw = ramselw ? ram : kernelenw ? krnl : basicenw ? basc : char;
assign cpu_PR = ~vic_r_reg ? DBRvic: ~sid_r_reg ? DBRsid : ~cia1_r_reg ? DBRcia1 : ~cia2_r_reg ? DBRcia2 : ~io1_r_reg ? DBRio1 : ~io2_r_reg ? DBRio2 : { 4'h0, DBRcolor};

assign pRead = vicr | sidr | cia1r | cia2r | io1r | io2r | colorr;

assign vic_r_reg   = ~vicr;
assign sid_r_reg   = ~sidr;
assign cia1_r_reg  = ~cia1r;
assign cia2_r_reg  = ~cia2r;
assign io1_r_reg   = ~io1r;
assign io2_r_reg   = ~io2r;

assign vic_w_reg  = ~vicw;
assign sid_w_reg  = ~sidw;
assign cia1_w_reg = ~cia1w;
assign cia2_w_reg = ~cia2w;
assign io1_w_reg  = ~io1w;
assign io2_w_reg  = ~io2w;

assign dAB = dAB1;

//#################################################          
// data read selection
always@(posedge mclk)
  begin

    vic_bitmap <= ~vic_charRom ? char : ram;

    dma_ack2 <= dma_ack;
    dma_ack3 <= dma_ack2;
    
    dma_dr <= dma_dr2;
    
    if(~dma_ack)
      begin
        //only update this register when there is an actual dma access
        if(pRead) dma_dr2 <= cpu_PR;
        else
          case(cpu_awl[1:0])
            2'h0 : dma_dr2 <= cpu_DRw[7:0];
            2'h1 : dma_dr2 <= cpu_DRw[15:8];
            2'h2 : dma_dr2 <= cpu_DRw[23:16];
            2'h3 : dma_dr2 <= cpu_DRw[31:24];
          endcase
      end
  end

endmodule
